/**********************************************************/
/*                    IBM Confidential                    */
/*      (IBM Confidential-Restricted when combined        */
/*       with the Aggregated OCO Source Modules for       */
/*       this Program)                                    */
/*                                                        */
/*                 OCO Source Materials                   */
/*                                                        */
/*        IBM AIX Speech Server/6000    Release 1.0       */
/*          5777-WCQ (C) Copyright IBM Corp. 1992         */
/**********************************************************/
/*=====================================================================*/
/*                                                                     */
/*  smenr.h                                                            */
/*                                                                     */
/*  private header for the enrollment functions                        */
/*                                                                     */
/*=====================================================================*/

/* $Header: /tanglite/cvsroot/vv/api/smenr.h,v 1.11 1998/03/19 21:29:43 prosser Exp $
 *
 * $Log: smenr.h,v $
 * Revision 1.11  1998/03/19 21:29:43  prosser
 * Wrap SmSes... prototypes with extern "C".
 *
 * Revision 1.10  1998/02/27  18:51:09  prosser
 * Added support for SmRemoveUser and SmRemoveEnrollId.
 *
 * Revision 1.9  1998/02/18  15:46:28  prosser
 * Added sample_rate argument to Sm(Ses)AddEnrollid.
 *
 * Revision 1.8  1997/08/15  17:38:49  prosser
 * Modified SmQueryScripts prototype.  Added prototypes for SmGetScriptFlags,
 * SmGetIncrements.
 *
 * Revision 1.7  1996/08/05  14:12:25  prosser
 * cmvc #8669 - Moved SmGetEnrollPath, SmGetNumberProcessed,
 * SmGetNumberRequired, SmGetNumberUtterances, and SmGetScript from smmsg.h.
 *
 * Revision 1.6  95/08/05  22:43:40  degenn
 * API level 3.5: added SmAddEnrollid, minor changes in logging (window ids in hex...)
 * 
 * Revision 1.5  95/06/04  14:21:25  degenn
 * Made SmSes* calls public, dropped smsapi.h, dropped public sm_errno.
 * 
 * Revision 1.4  95/05/31  16:28:44  degenn
 * Support for deferred correction.
 * 
 * Revision 1.3  95/02/06  15:38:19  c1farooq
 * Made the changes corresponding to API level "3.1a".
 * 
 * Revision 1.2  95/01/31  15:41:09  c1farooq
 * Moved the enrollment related functions from public headers
 * to enrollment specific header 'smenr.h'.
 * 
 * Revision 1.1  95/01/17  14:47:16  c1farooq
 * Initial revision
 * 
 *
 */

#ifndef SM_ENR_H_INCLUDED
#define SM_ENR_H_INCLUDED

/*-------------------------------------------------------------------------*/
/* speech manager includes to handle arguments, communication and access   */
/* to reply messages                                                       */
/*-------------------------------------------------------------------------*/
#include <smapi.h>

/*-------------------------------------------------------------------------*/
/* requested services                                                      */
/*-------------------------------------------------------------------------*/
#define SmNenrollment                  "enrollment"
#define SmNqueryEnrollment             "queryEnrollment"

/*-------------------------------------------------------------------------*/
/* selection of an enrollment script                                       */
/*-------------------------------------------------------------------------*/
#define SmNscript                      "script"

/*------------------------------------------------------------------------*/
/* put a wrapper around these functions so that they can be called from   */
/* either C or C++ applications                                           */
/*------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

/*-------------------------------------------------------------------------*/
/* enrollment functions                                                    */
/*-------------------------------------------------------------------------*/
int SmAddUser             ( char          *user_id,
			    char          *user_name,
			    char          *description,
                            char          *password,
			    SM_MSG        *reply );

int SmAddEnrollId         ( char          *user_id,
			    char          *enroll_id,
			    char          *description,
                            char          *language,
                            short          sample_rate,
			    SM_MSG        *reply );

int SmRemoveUser          ( char          *user_id,
			    SM_MSG        *reply );

int SmRemoveEnrollId      ( char          *user_id,
			    char          *enroll_id,
			    SM_MSG        *reply );

int SmRequestNewEnrollId  ( char          *user_id,
			    char          *user_name,
			    SM_MSG        *reply );

int SmQueryScripts        ( char          *user_id,
                            char          *enrollid,
                            char          *task,
			    SM_MSG        *reply );

int SmRequestScriptText   ( long           uttno,
			    SM_MSG        *reply );

int SmSelectScript        ( char          *scriptname,
			    char          *language,
			    SM_MSG        *reply );

int SmQueryUtterances     ( SM_MSG        *reply );

int SmSetUtteranceNumber  ( long           uttno,
			    SM_MSG        *reply );

int SmDiscardUtterance    ( long           uttno,
			    SM_MSG        *reply );

/*-------------------------------------------------------------------------*/
/* the access functions associated only with the above enrollment messages */
/*-------------------------------------------------------------------------*/
int SmGetUtteranceList    ( SM_MSG          reply,
			    unsigned long  *nutterances,
			    long          **utterances);

int SmGetComment          ( SM_MSG          reply,
			    char          **comment);

int SmGetNumberProcessed  ( SM_MSG reply, unsigned long  *nprocessed     );
int SmGetNumberRequired   ( SM_MSG reply, unsigned long  *nrequired      );
int SmGetNumberUtterances ( SM_MSG reply, unsigned long  *nutterances    );

/*-------------------------------------------------------------------------*/
/* the enrollment related functions                                        */
/*-------------------------------------------------------------------------*/
int SmGetEnrollPath             ( SM_MSG reply, char **enroll_path               );
int SmGetExpectedRecordingSpace ( SM_MSG sm_reply, unsigned long *expected_space );
int SmGetExpectedTrainingSpace  ( SM_MSG sm_reply, unsigned long *expected_space );
int SmGetFreeSpace              ( SM_MSG sm_reply, unsigned long *free_space     );
int SmGetRequiredTrainingSpace  ( SM_MSG sm_reply, unsigned long *required_space );
int SmGetScript                 ( SM_MSG reply, char          **scripts          );
int SmGetScriptFlags            ( SM_MSG reply, unsigned long *nflags, 
                                  long **flags  );
int SmGetIncrements             ( SM_MSG reply, unsigned long  *nincrements,
                                  short **increments );


#ifdef __cplusplus
   }
#endif
/*-------------------------------------------------------------------------*/
/* ...and the corresponding callback flags                                 */
/*-------------------------------------------------------------------------*/
#define SmNaddUserCallback             "addUserCallback"
#define SmNaddEnrollIdCallback         "addEnrollIdCallback"
#define SmNremoveUserCallback          "removeUserCallback"
#define SmNremoveEnrollIdCallback      "removeEnrollIdCallback"
#define SmNrequestNewEnrollIdCallback  "requestNewEnrollIdCallback"
#define SmNqueryScriptsCallback        "queryScriptsCallback"
#define SmNrequestScriptTextCallback   "requestScriptTextCallback"
#define SmNselectScriptCallback        "selectScriptCallback"
#define SmNqueryUtterancesCallback     "queryUtterancesCallback"
#define SmNsetUtteranceNumberCallback  "setUtteranceNumberCallback"
#define SmNdiscardUtteranceCallback    "discardUtteranceCallback"

/*------------------------------------------------------------------------*/
/* Internal speech messages to/from applications                          */
/*------------------------------------------------------------------------*/
  /* Application to recognizer */
#define SM_ADD_USER_REQUEST             127
#define SM_REQUEST_NEW_ENROLLID         128
#define SM_QUERY_SCRIPTS                129
#define SM_REQUEST_SCRIPT_TEXT          130
#define SM_SELECT_SCRIPT                131
#define SM_QUERY_UTTERANCES             132
#define SM_SET_UTTERANCE_NUMBER         133
#define SM_DISCARD_UTTERANCE            134
  /* Recognizer to application */
#define SM_ADD_USER_REPLY               135
#define SM_REQUEST_NEW_ENROLLID_REPLY   136
#define SM_QUERY_SCRIPTS_REPLY          137
#define SM_REQUEST_SCRIPT_TEXT_REPLY    138
#define SM_SELECT_SCRIPT_REPLY          139
#define SM_QUERY_UTTERANCES_REPLY       140
#define SM_SET_UTTERANCE_NUMBER_REPLY   141
#define SM_DISCARD_UTTERANCE_REPLY      142

/*------------------------------------------------------------------------*/
/* Messages added after 3.1, added here to keep backward compatibility.   */
/*------------------------------------------------------------------------*/
#define SM_ADD_ENROLLID                 176
#define SM_ADD_ENROLLID_REPLY           177


/*------------------------------------------------------------------------*/
/* Entry points for parallel sessions.                                    */
/*------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

int SmSesAddUser            ( SmSesId        session_id,
			      char          *user_id,
			      char          *user_name,
			      char          *description,
			      char          *password,
			      SM_MSG        *reply );

int SmSesAddEnrollId        ( SmSesId        session_id,
			      char          *user_id,
			      char          *enroll_id,
			      char          *description,
                              char          *language,
			      short          sample_rate,
			      SM_MSG        *reply );

int SmSesRemoveUser         ( SmSesId        session_id,
			      char          *user_id,
		   	      SM_MSG        *reply );

int SmSesRemoveEnrollId     ( SmSesId        session_id,
			      char          *user_id,
			      char          *enroll_id,
			      SM_MSG        *reply );

int SmSesRequestNewEnrollId ( SmSesId        session_id,
			      char          *user_id,
			      char          *user_name,
			      SM_MSG        *reply );

int SmSesQueryScripts       ( SmSesId        session_id,
                              char          *user_id,
                              char          *enrollid,
                              char          *task,
			      SM_MSG        *reply );

int SmSesRequestScriptText  ( SmSesId        session_id,
			      long           uttno,
			      SM_MSG        *reply );

int SmSesSelectScript       ( SmSesId        session_id,
			      char          *scriptname,
			      char          *language,
			      SM_MSG        *reply );

int SmSesQueryUtterances    ( SmSesId        session_id,
			      SM_MSG        *reply );

int SmSesSetUtteranceNumber ( SmSesId        session_id,
			      long           uttno,
			      SM_MSG        *reply );

int SmSesDiscardUtterance   ( SmSesId        session_id,
			      long           uttno,
			      SM_MSG        *reply );

#ifdef __cplusplus
   }
#endif


#endif /* not SM_ENR_H_INCLUDED */
