/*================================================================*/
/* COPYRIGHT:                                                     */
/* ----------                                                     */
/* Copyright (C) International Business Machines Corp., 1996.     */
/*                                                                */
/* DISCLAIMER OF WARRANTIES:                                      */
/* -------------------------                                      */
/* The following [enclosed] code is sample code created by IBM    */
/* Corporation.  This sample code is not part of any standard IBM */
/* product and is provided to you solely for the purpose of       */
/* assisting you in the development of your applications.  The    */
/* code is provided "AS IS", without warranty of any kind.  IBM   */
/* shall not be liable for any damages arising out of your use of */
/* the sample code, even if they have been advised of the         */
/* possibility of such damages.                                   */
/*================================================================*/

#define STRICT
#include <windows.h>
#include <mmsystem.h>
#include <stdio.h>
#include <stdlib.h>



#include "audwav.h"
#include "waves.h"

// NEW-PLAYBACK

#include "spchrc.h"
#include "pcmhead.h"

// NEW-PLAYBACK


/*-------------------------------------------------------------------*/
/* These calls are being made from a C++ program, so we need to use  */
/* their calling convention (as opposed to just C (__cdecl)          */
/*-------------------------------------------------------------------*/
#define CALL_CONV  __stdcall


// NEW-PLAYBACK

#define FIRST_TIME     0xFFFF
#define AUDIO_COUNTP   24     // # of playback audio buffers (filled by driver)
#define BUFFER_LENGTH  1024  // # of bytes in each buffer
#define PLAYBACK_MULTIPLIER_THRESHOLD  16384  // if 1st Put_PCM samples are less than this multiply by 2

#define BUFFER_WORDS  (BUFFER_LENGTH/2) // NEW FOR AUDIO METER

//
// When MME is used for recording only one audio channel is open.
// When Mwave is used, the record and playback channels can both be open.
//
static HGLOBAL     hWaveHdrP[AUDIO_COUNTP];    // handles of the wave play headers
static LPWAVEHDR   lpWaveHdrP[AUDIO_COUNTP];   // pointers to the wave play headers
static HGLOBAL     hDataP[AUDIO_COUNTP];       // handles of wave play data buffers

static int                playback_started;     // Set to 1 after 1st Put_PCM
static short              playback_multiplier;  // Multiplier of playback PCM to get it to be close to system levels
static HWAVEOUT           play_deviceID=0;
static int                prod_indexm;          // producer index
static int                cons_indexm;          // consumer index
static unsigned long      play_error;
static int                local_sample_rate;

#define       NO_RECORD_DEVICE    1
#define       NO_PLAYBACK_DEVICE  2

static int    con_problem=NO_RECORD_DEVICE;
static UINT   NumDevsP;      // # of wave-out (playback) audio devices
static UINT   OutputCard=FIRST_TIME;// playback wave audio device ID in registry
static UINT   DevIDP;               // playback wave audio device ID

char   ErrMsg[100];
short  initialized = 0;

// ----------------------------------------------------------------------------
//  Enumerate the possible audio channel modes.
// ----------------------------------------------------------------------------
enum Audio_Channel_State {
  Audio_Disconnected,
  Audio_Connected
  };

enum Audio_Processor_State {
  Audio_Idle,
  Audio_Recording,
  Audio_Playing
  };

//device states
enum Device_States {
   IDLE,
   OPEN,
   RECORDING,
   STOPPED,
   CLOSED,
   LOADING,
   PLAYING
};

static enum Audio_Channel_State    channel_state;
static enum Audio_Processor_State  processor_state;
static enum Device_States play_dev_state;       //  WAV out device state


ULONG close_play(void);
ULONG close_record(int);
ULONG open_play(void);
void  free_memory(void);
void  determine_play_device(void);
long  num_mt_bufmP(void);
ULONG test_open_playback(UINT DevIDP);
char * getreg ( char *env_var );

// NEW-PLAYBACK


/*-------------------------------------------------------------------*/
/* THE_AUDIO_SOURCE is the name of the audio file that we are going  */
/* to be decoding from.  It is set via the AudioSetSource call       */
/* which is called by the client via SmSetBinary)                    */
/*-------------------------------------------------------------------*/
static char the_audio_source [ 255 ] = { '\0' };

/*-------------------------------------------------------------------*/
/* THE_AUDIO_FORMAT and THE_SAMPLE_RATE are the format and sampling  */
/* rate that the recognizer is expecting the audio to be in (11KHz   */
/* linear PCM, for example)                                          */
/*-------------------------------------------------------------------*/
static int    the_audio_format  = 0;
static int    the_sample_rate   = 0;

       int    the_log_level     = 0;
       FILE * the_log_file      = NULL;


/*-------------------------------------------------------------------*/
/* AUDIO_FD is the file handle for the WAV file that we're currently */
/* 'recording' from                                                  */
/*-------------------------------------------------------------------*/
static int           audio_fd   = -1;

static int           block_num = 0;

static char          log_buff [ 255 ];



/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioStopRecording ( void )
{
  DebugMsg ( "AudioStopRecording: invoked\n" );

  WavCloseFile ( audio_fd );

  audio_fd = -1;
// NEW-PLAYBACK
  processor_state = Audio_Idle;
// NEW-PLAYBACK

  return ( 0 );
}

/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioCreate ( const char * file_name,
                            FILE       * log_file,
                            int          log_level )
{
   the_log_file   = log_file;

  DebugMsg ( "AudioCreate: invoked\n" );

  the_log_level  = log_level;
// NEW-PLAYBACK
  initialized   = 0;
// NEW-PLAYBACK

  return ( 0 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
void CALL_CONV AudioDestroy ( void )
{
  DebugMsg ( "AudioDestroy: invoked\n" );

  the_log_file = NULL;

  the_log_level = 0;

  the_sample_rate  = 0;

  the_audio_format = 0;

  audio_fd = -1;
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioConnect ( const char   * init_string,
                             const char   * client_name,
                             int          * handle,
                             int            sample_rate,
                             int            fformat,
                             int          * data_size,
                             int          * byte_order,
                             const char * * signature,
                             char         * cepslan_data )
{
  short i;
  DebugMsg ( "AudioConnect: invoked\n" );

  /*----------------------------------------------------------------------*/
  /* INIT_STRING is not used in this sample.  It's the last value of the  */
  /* string the client passed in as SmNaudioHost when it connected to the */
  /* engine.                                                              */
  /*----------------------------------------------------------------------*/
  sprintf ( log_buff, "AudioConnect: InitString = '%s'\n", init_string );
  DebugMsg ( log_buff );


  /*----------------------------------------------------------------------*/
  /* THE_SAMPLE_RATE and THE_AUDIO_FORMAT are the rate and format the     */
  /* engine is going to be expecting data in (eg 11KHZ, 16-bit mono PCM)  */
  /* These values should be compared to what'sin the WAV file header when */
  /* it's opened later                                                    */
  /*----------------------------------------------------------------------*/
  the_sample_rate  = sample_rate;

  the_audio_format = fformat;


  * handle = -1;

  /*----------------------------------------------------------------------*/
  /* DATA_SIZE and BYTE_ORDER are used by the engine to interpret what we */
  /* are sending back in its audio buffers                                */
  /*----------------------------------------------------------------------*/
  * data_size = BLOCK_DATA_SIZE;

  * byte_order = AUDIO_LITTLE_ENDIAN;


  * signature = "WAV11   ";


// NEW-PLAYBACK

  OutputCard    = FIRST_TIME;
  local_sample_rate = sample_rate;

  if (initialized)
  {
     close_play();
     AudioStopRecording();
     free_memory();
     initialized = 0;
  }


  con_problem = 0;
  determine_play_device();

  switch (con_problem) {
    case NO_PLAYBACK_DEVICE:
      LogMsg( "AUDMME! Connect failed: no audio playback device!");
      return -1;

    default:
      break;
  }

  play_dev_state = CLOSED;
  prod_indexm    = 0;
  cons_indexm    = 0;
  play_error     = 0;


  // --------------------------------------------------------------------------
  // Allocate the playback wave i/o headers and buffers
  // --------------------------------------------------------------------------
  for (i = 0; i < AUDIO_COUNTP; i++)
  {
    // Allocate a wave header
    if ((hWaveHdrP[i] = GlobalAlloc(GMEM_MOVEABLE | GMEM_SHARE, sizeof(WAVEHDR))) == NULL)
    {
      sprintf(ErrMsg,"%-24s %s 0x%x", "Error: AudMME!", "Cannot Alloc a header");
      LogMsg(ErrMsg);
      return -1;
    }

    if ((lpWaveHdrP[i] = (LPWAVEHDR)GlobalLock(hWaveHdrP[i])) == NULL){
      sprintf(ErrMsg,"%-24s %s 0x%x", "Error: AudMME!", "Cannot Lock a header");
      LogMsg(ErrMsg);
      return -1;
    }

    // Allocate a wave data buffer
    if ((hDataP[i] = GlobalAlloc(GMEM_MOVEABLE | GMEM_SHARE, BUFFER_LENGTH)) == NULL)
    {
      sprintf(ErrMsg, "%-24s %s 0x%x", "Error: AudMME!", "Cannot Alloc a data buffer");
      LogMsg(ErrMsg);
      return -1;
    }

    if ((lpWaveHdrP[i]->lpData = (LPSTR)GlobalLock(hDataP[i])) == NULL)
    {
      sprintf(ErrMsg,"%-24s %s 0x%x", "Error: AudMME!", "Cannot Lock a data buffer");
      LogMsg(ErrMsg);
      return -1;
    }

    // Initialize wave header
    lpWaveHdrP[i]->dwBufferLength = BUFFER_LENGTH;
    lpWaveHdrP[i]->dwFlags        = 0;
    lpWaveHdrP[i]->dwLoops        = 0;
    lpWaveHdrP[i]->lpNext         = 0;
    lpWaveHdrP[i]->reserved       = 0;
  }

  // --------------------------------------------------------------------------
  //  Indicate that everything went well.
  // --------------------------------------------------------------------------
  channel_state = Audio_Connected;

  initialized = 1;
  processor_state = Audio_Idle;

// NEW-PLAYBACK


  DebugMsg ( "AudioConnect: done\n" );

  return ( 0 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioDisconnect ( void )
{

  DebugMsg ( "AudioDisconnect: invoked\n" );

  if ( audio_fd )
  {
    AudioStopRecording();
// NEW-PLAYBACK
    close_play();
// NEW-PLAYBACK
  }


// NEW-PLAYBACK

  // --------------------------------------------------------------------------
  //  There's nothing else to do, except indicate the new mode.
  // --------------------------------------------------------------------------
  channel_state = Audio_Disconnected;
  processor_state = Audio_Idle;
// NEW-PLAYBACK

  return ( -1 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioGetHandle ( void )
{
  DebugMsg ( "AudioGetHandle: invoked\n" );

  return ( audio_fd );
}



/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioSetSource ( short len, void * value )
{
   DebugMsg ( "AudioSetSource: invoked\n" );

   sprintf ( log_buff, "AudioSetSource: source = '%s'\n", ( char * ) value );
   LogMsg ( log_buff );

   //JZ***7/13/00
   memset ( the_audio_source, '\0', sizeof ( the_audio_source ) );

   /*----------------------------------------------------------------------*/
   /* Copy the name of the file that the client has passed to our local    */
   /* buffer so we can use it later (in AudioStartRecording)               */
   /*----------------------------------------------------------------------*/
   //JZ***7/13/00
   //strcpy ( the_audio_source, ( char * )value );
   strncpy ( the_audio_source, ( char * ) value, len );

   return( 0 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioQuerySource ( short * len, void * * value )
{
  DebugMsg ( "AudioQuerySource: invoked\n" );

  /*----------------------------------------------------------------------*/
  /* Return whatever we have currently as 'the_audio_source' to the       */
  /* client                                                               */
  /*----------------------------------------------------------------------*/
  * len = strlen ( the_audio_source );

  * value = the_audio_source;

  return ( 0 );
}



/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioStartRecording ( unsigned long * time_zero,
                                    int             report_errors )
{
  int rc;

  DebugMsg ( "AudioStartRecording: invoked\n" );

#if 0
  // Test for the correct wave format
  if ((rc = WavCheckFormat( the_audio_source, the_sample_rate )) != 0)
     return( rc );
#endif

  sprintf ( log_buff, "AudioStartRecording: File = %s\n",the_audio_source);
  DebugMsg ( log_buff );

  /*----------------------------------------------------------------------*/
  /* Open the wav file (see 'waves.c' for how this is done)               */
  /*----------------------------------------------------------------------*/
  if ( ( audio_fd = WavOpenFile ( the_audio_source ) ) < 0 )
  {
    sprintf ( log_buff,
              "AudioStartRecording: WavOpenFailed for '%s', rc = %d\n",
              the_audio_source, audio_fd );

    ErrorMsg ( log_buff );

    sprintf( log_buff, "Unable to open input file %s", the_audio_source );
    MessageBox( NULL, log_buff, "ENGINE.EXE - AUDMME.DLL", MB_TOPMOST );

    return ( -999 );
  }

  DebugMsg ( "AudioStartRecording: file opened\n" );


  /*----------------------------------------------------------------------*/
  /* Position the file pointer to the first byte of the first block of    */
  /* data within the wav file                                             */
  /*----------------------------------------------------------------------*/
  if ( ( rc = WavFirstDataBlock ( audio_fd ) ) < 0 )
  {
    sprintf ( log_buff,
              "AudioStartRecording: WavOpenFailed for '%s', rc = %d\n",
              the_audio_source, audio_fd );

    ErrorMsg ( log_buff );

    return ( -998 );
  }


  DebugMsg ( "AudioStartRecording: ready to go\n" );

  * time_zero = GetTickCount ( ) / 10;

  sprintf ( log_buff, "AudioStartRecording: time = %d\n", * time_zero );

  DebugMsg ( log_buff );

  // --------------------------------------------------------------------------
  //  Indicate that we are recording.
  // --------------------------------------------------------------------------
  processor_state      = Audio_Recording;

  return ( 0 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioGetPCM ( char * block_buffer,
                            long   max_bytes,
                            long * new_bytes,
                            int  * end_of_pcm )
{
  int     bytes_to_send;
  int     done;
  int     head_pos;
  int     data_pos;
  int     bytes_done;
  int     rc, i;
  short * header;
  int     num_blocks;
  short   *sample_buffer;
  register short   volume;

  DebugMsg ( "AudioGetPCM: invoked\n" );

  * end_of_pcm = FALSE;

  /*----------------------------------------------------------------------*/
  /* Figure out how many bytes we can send back - the most we can send is */
  /* 'max_bytes', which is how much the engine has room for right now     */
  /*----------------------------------------------------------------------*/
  bytes_to_send = max_bytes;

  /*----------------------------------------------------------------------*/
  /* And calculate how many blocks that is.  If it's less than 1, then    */
  /* what...                                                              */
  /*----------------------------------------------------------------------*/
  num_blocks = bytes_to_send / BLOCK_TOTAL_SIZE;


  sprintf ( log_buff, "AudioGetPcm: bytes_2send = %d\n", bytes_to_send );
  DebugMsg ( log_buff );

  /*----------------------------------------------------------------------*/
  /* Now start reading blocks of data from the WAV file until we either   */
  /* run out of data (reach the end of the file), or fill up the engine's */
  /* buffers                                                              */
  /*----------------------------------------------------------------------*/
  bytes_done = 0;
  head_pos = 0;
  data_pos = BLOCK_HEADER_SIZE;

  done = FALSE;

  while ( ( ! done ) && ( num_blocks > 0 ) )
  {
    rc = WavReadBlock ( audio_fd, & block_buffer [ data_pos ], BLOCK_DATA_SIZE );

    bytes_done = bytes_done + BLOCK_HEADER_SIZE + rc;

    if ( rc != BLOCK_DATA_SIZE )
    {
      * end_of_pcm = TRUE;

      done = TRUE;
    }

    sprintf ( log_buff, "AudioGetPcm: eof = %d, siz = %d, bd = %d\n",
              * end_of_pcm, rc, bytes_done );
    DebugMsg ( log_buff );

    /*----------------------------------------------------------------------*/
    /* Each block of data being sent back to the engine (one call to this   */
    /* function returns multiple contiguous blocks!!!) has a fixed size     */
    /* header in it, describing the contents of that block                  */
    /*----------------------------------------------------------------------*/
    header = ( short * ) & block_buffer [ head_pos ];

// NEW FOR AUDIO LEVEL

    volume = 0;
    sample_buffer = (short *) &block_buffer[data_pos];
    for ( i = 0; i < (int)(rc / sizeof(short)); i = i + 4 )
    {
      if ( abs(sample_buffer[i]) > volume )
      {
        volume = abs(sample_buffer[i]);
      }
    }

    sprintf ( log_buff, "AudioGetPcm: Volume = %d\n",volume);
    DebugMsg ( log_buff );


// NEW FOR AUDIO LEVEL



    memset ( header, '\0', BLOCK_HEADER_SIZE );

    header [ HEADER_OFFSET_BYTE_ORDER  ] = 0x0102;
    header [ HEADER_OFFSET_FORMAT      ] = 2;
    header [ HEADER_OFFSET_SAMPLES     ] = rc / sizeof(short); //BLOCK_DATA_SIZE / sizeof ( short );
    header [ HEADER_OFFSET_VOLUME      ] = volume;
    header [ HEADER_OFFSET_ONE         ] = 1;
    header [ HEADER_OFFSET_BLOCK_INDEX ] = block_num++;
    header [ HEADER_OFFSET_END_OF_PCM  ] = ( * end_of_pcm );
    header [ HEADER_OFFSET_PCM_LENGTH  ] = rc; BLOCK_DATA_SIZE;
    header [ HEADER_OFFSET_DATA_LENGTH ] = rc;

    /*----------------------------------------------------------------------*/
    /* Positon HEAD_POS and DATA_POS at the start of the next header and    */
    /* data block                                                           */
    /*----------------------------------------------------------------------*/
    head_pos = head_pos + BLOCK_TOTAL_SIZE;
    data_pos = data_pos + BLOCK_TOTAL_SIZE;

    num_blocks = num_blocks - 1;
  }


  * new_bytes = bytes_done;

  return ( 0 );
}






/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioStartPlayback ( void )
{
  int   i,rc;

  DebugMsg ( "AudioStartPlayback: invoked\n" );

  LogMsg ( "AudioStartPlayback: invoked\n" );

// NEW-PLAYBACK
  if (channel_state != Audio_Connected)
  {
    return SPCH_RC_AUDIO_DISCONNECTED;
  }

  if ( processor_state == Audio_Playing )
  {
    if ((rc = close_play()) != 0)
    {
      sprintf(ErrMsg,"%-24s %s 0x%x\n", "AudMME!", "Start_Playback, close_play rc=0x%x",rc);
      LogMsg(ErrMsg);
      return SPCH_RC_PLAY_OPEN_ERROR;
    }
  }

  if ( processor_state == Audio_Recording )
  {
    AudioStopRecording ( );
  }


  if (play_dev_state == CLOSED)
  {

    DebugMsg("Start_Playback: opening device\n");

    determine_play_device();

    for (i=0; i<10; i++)
    {
      // try opening 10 times (about one second) before giving up
      if ((rc = open_play()) == 0)
        goto open_play_successful;

      Sleep(100);

    }

    // open_play unsuccessful

    sprintf(ErrMsg,"%-24s %s rc=0x%x\n", "AudMME!", "Start_Playback open_play,", rc);
    LogMsg(ErrMsg);
    return SPCH_RC_PLAY_OPEN_ERROR;

open_play_successful:

    for (i = 0; i < AUDIO_COUNTP; i++) {
      lpWaveHdrP[i]->dwFlags = 0;WHDR_DONE;  // clear flags
      lpWaveHdrP[i]->dwLoops = 0;
      lpWaveHdrP[i]->dwBufferLength = BUFFER_LENGTH;

      if (( rc = waveOutPrepareHeader(play_deviceID, lpWaveHdrP[i], sizeof(WAVEHDR))) != 0)
      {
        sprintf(ErrMsg,"%-24s %s rc=0x%x\n", "AUDMME!", "Start_Playback waveOutPrepare", rc);
        LogMsg(ErrMsg);
        return SPCH_RC_PLAY_OPEN_ERROR;
      }
    }
  }

  play_error = 0;
  cons_indexm = 0;
  prod_indexm = 0;
  playback_started = 0;
  playback_multiplier = 1;



  return ( 0 );


// NEW-PLAYBACK
}


int CALL_CONV AudioPutPCM ( char * BlockBuffer,
                            int    BlockBufferByteCnt )
{
   int BlockBufferIndex, CurBlockByteCnt, AudioBufferCnt;
   int i, rc, AudioBufferIndex, SampleCnt;
   int BlockBufferWordCnt;
   int AudioBytesProcessed;
   short *pwBlockBuffer = (short *)BlockBuffer;
   short *pwAudioBuffer;

   BlockBufferWordCnt = BlockBufferByteCnt / 2;
   AudioBytesProcessed = 0;

   // If the audio is not connected, just bail out
   if (channel_state != Audio_Connected)
      return 0; // do nothing if audio is disconnected.

   if (BlockBufferByteCnt == 0)
   {
      if (num_mt_bufmP() == AUDIO_COUNTP)
      {
         // all buffers are empty
         play_dev_state = STOPPED;
         return 0;
      }
      else
      {
         Sleep(BUFFER_LENGTH/44); // Sleep half the time it takes to play a buffer
         return(1);         // 1 to indicate 'play in progress'
      }
   }

   // Start at beginning of data buffer
   BlockBufferIndex = 0;

   while( BlockBufferIndex < BlockBufferWordCnt )
   {
      // We start with the index pointing to a header, so get the
      // number of actual data bytes for this block...
      CurBlockByteCnt = pwBlockBuffer[BlockBufferIndex+PCM_HEADER_OFFSET_PCM_LENGTH];

      // Move index past the header to the data..
      BlockBufferIndex += PCM_HEADER_LENGTH / 2;

      // Calculate the number of audio buffers needed to process this block
      AudioBufferCnt = CurBlockByteCnt / BUFFER_LENGTH + 1;

      for( AudioBufferIndex = 0; AudioBufferIndex < AudioBufferCnt; AudioBufferIndex++ )
      {
         pwAudioBuffer = (short *)(lpWaveHdrP[prod_indexm]->lpData);

         if( (CurBlockByteCnt - (AudioBufferIndex*BUFFER_LENGTH)) <= BUFFER_LENGTH )
            SampleCnt = (CurBlockByteCnt - (AudioBufferIndex*BUFFER_LENGTH)) / 2;
         else
            SampleCnt = BUFFER_LENGTH;

         if( SampleCnt == 0 )
            continue;

         if ((num_mt_bufmP()) < 6)
         {
            Sleep ((BUFFER_LENGTH*6)/22 + 50); // sleep enough to empty a buffer
         }
         if (num_mt_bufmP() < 6 )
         {
            // play buffers don't empty fast enough
            //_asm int 3;
            Sleep ((BUFFER_LENGTH*6)/22 + 50);
            return( AudioBytesProcessed );
         }

         AudioBytesProcessed += SampleCnt * 2;

         for( i = 0; i < SampleCnt; i++ )
            pwAudioBuffer[i] = pwBlockBuffer[BlockBufferIndex++];

         lpWaveHdrP[prod_indexm]->dwFlags &= ~WHDR_DONE;
         lpWaveHdrP[prod_indexm]->dwBufferLength = SampleCnt*2;

         if ((rc = waveOutWrite(play_deviceID, lpWaveHdrP[prod_indexm], sizeof(WAVEHDR))) !=0)
         {
            sprintf(ErrMsg,"%-24s %s 0x%x", "AUDMME!", "Unable to transfer buffer to device, rc =", rc);
            LogMsg(ErrMsg);
            return 0;
         }
         prod_indexm = (prod_indexm + 1)%AUDIO_COUNTP;
         processor_state = Audio_Playing;
      }
   }

   return( AudioBytesProcessed );
}

/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioStopPlayback ( int flush_output )
{

ULONG  rc;
int    nfb;

  DebugMsg ( "AudioStopPlayback: invoked\n" );

  LogMsg ( "AudioStopPlayback: invoked\n" );

// NEW-PLAYBACK

  if (channel_state != Audio_Connected)
  {
    return SPCH_RC_AUDIO_DISCONNECTED;
  }


  if (processor_state == Audio_Playing)
  {
         rc = 0;  // initialize

//
// The Turtle Beach card does not process correctly waveOutReset. The only
// way to make this card work is to wait for all the queued play buffers (up to
// AUDIO_COUNTP (8) to play before resetting, unpreparing and closing play.
//

         // wait for outstanding buffers to play, then close
         nfb = AUDIO_COUNTP - num_mt_bufmP();

         sprintf(ErrMsg,"Stop_Playback #full buffers=%d\n",nfb);
         DebugMsg (ErrMsg);

         while (num_mt_bufmP() < AUDIO_COUNTP && nfb >= 0)
         {
           Sleep(BUFFER_LENGTH/22);
           sprintf(ErrMsg,"Stop_Playback waiting ...\n");
           DebugMsg (ErrMsg);
           nfb--;
         }
         rc = close_play();

     if (rc != 0)
     {
        sprintf(ErrMsg,"%-24s %s rc=0x%x", "AUDMME!", "Can't stop playback", rc);
        LogMsg(ErrMsg);
        return (-1);
     }

     processor_state = Audio_Idle;
  }


  return ( 0 );

// NEW-PLAYBACK
}




/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioQueryDevices ( long * devices )
{
  DebugMsg ( "AudioQueryDevices: invoked\n" );

  LogMsg ( "AudioQueryDevices: invoked\n" );

  return ( -1 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioSetDevice ( long device )
{
  DebugMsg ( "AudioSetDevices: invoked\n" );

  LogMsg ( "AudioSetDevices: invoked\n" );

  return ( -1 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioQueryConfig ( long * configuration )
{
  DebugMsg ( "AudioQueryConfig: invoked\n" );

  LogMsg ( "AudioQueryConfig: invoked\n" );

  return ( -1 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioSetInput ( long input_device )
{
  DebugMsg ( "AudioSetInput: invoked\n" );

  LogMsg ( "AudioSetInput: invoked\n" );

  return ( -1 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioSetInputGain ( long input_gain )
{
  DebugMsg ( "AudioSetInputGain: invoked\n" );

  LogMsg ( "AudioSetInputGain: invoked\n" );

  return ( -1 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioSetOutput ( long output_gain )
{
  DebugMsg ( "AudioSetOutput: invoked\n" );

  LogMsg ( "AudioSetOutput: invoked\n" );

  return ( -1 );
}


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*---------------------------------------------------------------------------*/
int CALL_CONV AudioSetOutputGain ( long output_gain )
{
  DebugMsg ( "AudioSetOutputGain: invoked\n" );

  LogMsg ( "AudioSetOutputGain: invoked\n" );

  return ( -1 );
}




// NEW-PLAYBACK
// All the functions below are new for playback





// ============================================================================
//
// open_play()
//
// ============================================================================
ULONG open_play(void) {
   MMRESULT              rc;
   WAVEFORMATEX          play_open_parms;

   // -----------------------------------------------------------------------
   //  Open Device
   // -----------------------------------------------------------------------

   play_open_parms.wFormatTag      = WAVE_FORMAT_PCM;
   play_open_parms.nChannels       = 1;
   play_open_parms.nSamplesPerSec  = (ULONG)local_sample_rate;
   play_open_parms.nAvgBytesPerSec = (ULONG)(local_sample_rate * 2);
   play_open_parms.nBlockAlign     = 2;
   play_open_parms.wBitsPerSample  = 16;
   play_open_parms.cbSize          = 0;

   if ((rc = waveOutOpen((LPHWAVEOUT) &play_deviceID, DevIDP,
                    &play_open_parms, 0L, 0L, CALLBACK_NULL)) != MMSYSERR_NOERROR)
   {
      sprintf(ErrMsg,"open_play error rc=0x%x",rc);
      DebugMsg(ErrMsg);
      return rc;
   }

   play_dev_state = OPEN;
   return 0;
} //open_play

// ============================================================================
long num_mt_bufmP()
{


  if (cons_indexm == prod_indexm)
    return AUDIO_COUNTP;

  // update consumer
  while ( lpWaveHdrP[cons_indexm]->dwFlags & WHDR_DONE )
  {
    cons_indexm = (cons_indexm + 1)%AUDIO_COUNTP;
    if (cons_indexm == prod_indexm)
    {
      return AUDIO_COUNTP;
    }
  }

  if (prod_indexm >= cons_indexm)
     return AUDIO_COUNTP - prod_indexm + cons_indexm;
  else
     return cons_indexm - prod_indexm;
}

// ============================================================================
//
//  determine_play_device();
//
// ============================================================================
void determine_play_device(void)
{
  char             *prev;
  UINT              devid;

  // --------------------------------------------------------------------------
  // Determine play wave audio device only if registry entry changed
  // OutputCard = FIRST_TIME : this is the first time into this routine
  // OutputCard = FIRST_TIME+1 : not first time, & registry entry is NULL
  // --------------------------------------------------------------------------
  if (OutputCard != FIRST_TIME)
  {
    if ((prev = getreg ("SPCH_PLAY_ID")) != NULL)
    {
      devid = atol(prev);
      if (devid == OutputCard) // output device has already been determined
        return;
    }
    else
    {
      if ( OutputCard == FIRST_TIME + 1) // output device has already been determined
        return;
      sprintf(ErrMsg,"determine_play_device error: OutputCard not FIRST_TIME+1");
      LogMsg(ErrMsg);
    }
  }

  // --------------------------------------------------------------------------
  // Determine play wave audio device
  // --------------------------------------------------------------------------
  NumDevsP = waveOutGetNumDevs();


  sprintf(ErrMsg,"# of wave audio play devices = %d", NumDevsP);
  LogMsg(ErrMsg);


  // -----------------------------------------------------------------------
  // Check registry for play device ID
  //------------------------------------------------------------------------------
  DevIDP = (UINT)-1;
  if ((prev = getreg("SPCH_PLAY_ID"))!=NULL)
  {
    devid = atol(prev);
    OutputCard = devid;
    if (devid < NumDevsP && devid >= 0)
    {
      if (test_open_playback(devid)==0)
      {
        DevIDP = devid;
      }
      else
      {
        sprintf(ErrMsg,"Registry playback device %d can't be used!",devid);
        LogMsg(ErrMsg);
      }
    }
  }
  else
  {
   OutputCard = FIRST_TIME + 1;
  }

  if (DevIDP == -1)
  {
    for (DevIDP=0; DevIDP<NumDevsP;DevIDP++)
    {
      if(test_open_playback(DevIDP) == 0) break; // use first device that supports our format
    }
  }

  if (DevIDP == NumDevsP)
  {
      LogMsg("No playback device available!");
      con_problem = NO_PLAYBACK_DEVICE;
  }
}

// =============================================================================
//
// test_open_playback()
//
// =============================================================================
ULONG test_open_playback(UINT DevIDP)
{
   MMRESULT              rc;
   WAVEFORMATEX          rec_open_parms;

   // -------------------------------------------------------------------------
   //  Test Open Device
   // -------------------------------------------------------------------------

   rec_open_parms.wFormatTag      = WAVE_FORMAT_PCM;
   rec_open_parms.nChannels       = 1;
   rec_open_parms.nSamplesPerSec  = local_sample_rate;
   rec_open_parms.nAvgBytesPerSec = local_sample_rate * 2;
   rec_open_parms.nBlockAlign     = 2;
   rec_open_parms.wBitsPerSample  = 16;
   rec_open_parms.cbSize          = 0;

   if ((rc = waveOutOpen(NULL, DevIDP, &rec_open_parms,
                        0L, 0L, WAVE_FORMAT_QUERY)) != MMSYSERR_NOERROR){ // poll
      return rc;
   }

   return 0;

} //test_open_playback


// ============================================================================
//
// close_play()
//
// ============================================================================
ULONG close_play(void) {
   MMRESULT        rc;
   int i,count;

   if (play_dev_state == CLOSED)
      return 0; // return if device already closed

   DebugMsg("close_play: resetting ...");

   // -----------------------------------------------------------------------
   //  Stop then close playback
   // -----------------------------------------------------------------------
   // To handle Turtle Beach Stop_Recording must make sure all queued buffers
   // have played!

   if ((rc = waveOutReset(play_deviceID)) != MMSYSERR_NOERROR) // this will flush remaining play buffers
   {
     sprintf(ErrMsg,"%-24s %s 0x%x", "AudMME!", "close_play waveOutReset failed rc =", rc);
     LogMsg(ErrMsg);
     return rc;
   }

   for (i = 0; i < AUDIO_COUNTP; i++)
   {
       count = 0;
       while (( rc = waveOutUnprepareHeader(play_deviceID, lpWaveHdrP[i], sizeof(WAVEHDR))) == WAVERR_STILLPLAYING && count < AUDIO_COUNTP)
       {
         count++;
         Sleep (BUFFER_LENGTH/22);
         sprintf(ErrMsg,"close_play: waveOutUnprepare waiting ...i=%d",i);
         DebugMsg(ErrMsg);
       }
       if (rc != MMSYSERR_NOERROR)
       {
         sprintf(ErrMsg,"%-24s %s 0x%x", "AudMME!", "close_play Unprepare failed rc =", rc);
         LogMsg(ErrMsg);
         return rc;
       }
   }

   count = 0;
   while ((rc = waveOutClose(play_deviceID)) == WAVERR_STILLPLAYING && count < AUDIO_COUNTP)
   {
     count++;
     Sleep (BUFFER_LENGTH/22);

     sprintf(ErrMsg,"close_play: waveOutClose waiting ...count=%d",count);
     DebugMsg(ErrMsg);
   }

   if ( rc != MMSYSERR_NOERROR)
   {
       sprintf(ErrMsg,"%-24s %s rc=0x%x", "AudMME!", "close_play waveOutClose failed", rc);
       LogMsg(ErrMsg);
       return rc;
   }
   play_dev_state = CLOSED;

   DebugMsg("close_play: closed");

   return 0;

} //close_play


// ============================================================================
//
// free_memory ()
//
// ============================================================================
void free_memory(void)
{
int i;


  for (i = 0; i < AUDIO_COUNTP; i++)
  {
     GlobalUnlock(hWaveHdrP[i]);
     GlobalFree(hWaveHdrP[i]);
     GlobalUnlock(hDataP[i]);
     GlobalFree(hDataP[i]);
  }

}

char * getreg ( char *env_var )
{

   static char      value_ptr[ MAX_PATH + 1 ];

   HKEY      key_handle, key_root;
   LPCTSTR   subkey_name_ptr;
   LPTSTR    value_name_ptr;
   DWORD     value_size = MAX_PATH + 1;
   long      retcode;
   int       call_flag = 0;

   strupr( env_var );

   /*
       Check HKEY_LOCAL_MACHINE
   */

   key_root = HKEY_LOCAL_MACHINE;
   subkey_name_ptr = "SOFTWARE\\IBM\\VoiceType\\Engine\\Directories";

   /* Requesting value for SPCH_BIN                                    */
   if ( !strcmp( env_var, "SPCH_BIN" ) )  {
      value_name_ptr = "Bin";
      call_flag = 1;
      }

   /* Requesting value for SPCH_RO                                     */
   else if (!strcmp( env_var, "SPCH_RO" ) )  {
      value_name_ptr = "Vocabs";
      call_flag = 1;
      }

   /* Requesting value for SPCH_RW                                     */
   else if (!strcmp( env_var, "SPCH_RW" ) )  {
      value_name_ptr = "Users";
      call_flag = 1;
      }

   /* Requesting value for SPCH_RUN                                    */
   else if (!strcmp( env_var, "SPCH_RUN" ) )  {
      value_name_ptr = "Temp";
      call_flag = 1;
      }

   /* Requesting value for SPCH_TRN                                    */
   else if (!strcmp( env_var, "SPCH_TRN" ) )  {
      value_name_ptr = "Train";
      call_flag = 1;
      }

   /* Requesting value for SPCH_PATH                                   */
   else if (!strcmp( env_var, "SPCH_PATH" ) )  {
      value_name_ptr = "DataPath";
      call_flag = 1;
      }

#ifdef VTD30J
   /* Requesting value for SPCH_LOC                                    */
   else if (!strcmp( env_var, "SPCH_LOC" ) )  {
      value_name_ptr = "Location";
      call_flag = 1;
      }
#endif  /* VTD30J */

   if (call_flag == 0){
     /*
         Check Audio\General  (entries moved from user to machine)
     */

     subkey_name_ptr = "SOFTWARE\\IBM\\VoiceType\\Audio\\General";

     if (!strcmp( env_var, "SPCH_RECORD_ID" ) )  {
       value_name_ptr = "InputCard";
       call_flag = 1;
     } else if (!strcmp( env_var, "SPCH_PLAY_ID" ) )  {
       value_name_ptr = "OutputCard";
       call_flag = 1;
     } else if (!strcmp( env_var, "SPCH_MICORLINE" ) )  {
       value_name_ptr = "MicOrLine";
       call_flag = 1;
     } else if (!strcmp( env_var, "SPCH_DLAY" ) )  {
       value_name_ptr = "SpeechDelay";
       call_flag = 1;
     }
   }

   /* If call_flag was not set the environment variable data is not in */
   /* the windows registry.  Just call getenv.                         */
   if ( call_flag == 0 )
      return ( getenv ( env_var ) );


   if ( ( retcode = RegOpenKeyEx ( key_root,
                                   subkey_name_ptr, 0,
                                   KEY_QUERY_VALUE,
                                   &key_handle ) ) != ERROR_SUCCESS ) {
      if ( retcode == ERROR_FILE_NOT_FOUND )
         return ( getenv ( env_var ) );
      else
         return ( NULL );
      }


   if ( ( retcode = RegQueryValueEx ( key_handle,
                                      value_name_ptr,
                                      NULL, NULL,
                                      (LPBYTE)&value_ptr,
                                     &value_size ) ) != ERROR_SUCCESS ) {
      if ( retcode == ERROR_FILE_NOT_FOUND )
         return ( getenv ( env_var ) );
      else
         return ( NULL );
      }

   if ( ( retcode = RegCloseKey ( key_root ) ) != ERROR_SUCCESS )
      return ( NULL );
   else
      return ( value_ptr );


}

// NEW-PLAYBACK

