/*=====================================================================*/
/*                                                                     */
/*  spchrc.h - Include Speech API return codes.                        */
/*                                                                     */
/*  ----------                                                         */
/*                                                                     */
/*  IBM Confidential -- (IBM Confidential Restricted when combined     */
/*                       with the aggregated OCO source modules        */
/*                       for this program)                             */
/*                                                                     */
/*  OCO Source Materials ????-???                                      */
/*  (C) Copyright International Business Machine Corp, 1992.           */
/*  All Rights Reserved                                                */
/*                                                                     */
/*  US Government Users Restricted Rights - Use, duplication, or       */
/*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  */
/*                                                                     */
/*  ----------                                                         */
/*                                                                     */
/*=====================================================================*/

/* $Header: /u/prosser/publish/inc/RCS/spchrc.h,v 2.20 1997/04/25 21:24:08 prosser Exp $
 *
 * $Log: spchrc.h,v $
 * Revision 2.20  1997/04/25  21:24:08  prosser
 * Add SPCH_RC_RECORDING_REQUIRED.
 *
 * Revision 2.19  1996/09/09  17:51:41  prosser
 * cmvc ???? Add SPCH_RC_MISSING_EXTERN
 *
 * Revision 2.18  96/04/17  16:28:55  arvin
 * define a return code for adding a word that exists in extended vocabulay
 * 
 * Revision 2.17  95/06/04  16:59:51  degenn
 * Dropped separate native API level, since we now build one dll.
 * Since SPCHAPI_VERSION_STRING is used throughout code, tie it
 * to SM_API_VERSION_STRING.
 * 
 * Revision 2.16  95/05/10  14:01:27  daggett
 * Added SPCH_RC_NOT_WHILE_PLAYING return code.
 * 
 * Revision 2.15  95/05/04  09:18:57  sragh
 * Added new return code SPCH_RC_INVALID_WINDOW_HANDLE
 * 
 * Revision 2.14  95/02/07  11:40:16  c1farooq
 * Removed the new rc SM_RC_INVALID_AUDIO_DEVICE.
 *
 * Revision 2.13  95/02/06  16:42:57  c1farooq
 *  Added a new return code SM_RC_INVALID_AUDIO_DEVICE.
 *
 * Revision 2.12  95/01/17  15:23:15  c1farooq
 * Cleaned up the headers. Divided them into public and private sets.
 *
 * Revision 2.11  94/09/14  12:12:08  c1farooq
 * Bumped the API level to 3.0
 *
 * Revision 2.10  94/08/02  15:52:51  c1farooq
 * Added new return codes to inform client if it
 * is already opened for connection or is already
 * connected.
 *
 * Revision 2.9  94/07/15  12:15:17  c1farooq
 * Removed the unwanted return code 'SM_RC_NO_RECO_APPS'.
 *
 * Revision 2.8  94/06/27  17:17:04  c1farooq
 * Added return code for incompatible enrollment.
 *
 * Revision 2.7  94/06/22  09:24:45  c1farooq
 * Updated the api version.
 *
 * Revision 2.6  94/06/21  15:51:43  c1farooq
 * Added code for SmQuerySessions and SmDetachSessions.
 *
 * Revision 2.5  94/05/19  23:35:05  kushi
 * (David and Savita) Changes for persistent defaults,
 * focus swapping, etc.
 *
 * Revision 2.4  94/04/27  19:55:13  kushi
 * More session sharing enhancements.
 *
 * Revision 2.3  94/04/26  02:50:32  kushi
 * Bump API level to 2.0.2
 *
 * Revision 2.2  94/04/26  01:05:10  kushi
 * Session sharing phase II.
 *
 * Revision 2.1  94/03/22  12:12:44  kushi
 * AIX message queues and notification groups
 *
 * Revision 2.0  94/01/04  15:02:40  burn
 * Changed revision number only
 *
 * Revision 1.18  93/11/15  15:04:59  degenn
 * PTR 1568.
 * Set API version number to 2.0.0 for product release.
 *
 * Revision 1.17  93/09/10  19:43:43  kushi
 * changes to make the api flags and names consistent
 * bump the api level to 2.0.8 (PTAR 965).
 *
 * Revision 1.16  93/09/01  19:33:59  degenn
 * PTAR 337.
 * Changed audio input flags and audio configuration flags to
 * bit position values.  Bumped the SM and native API levels to
 * make sure the right values are sent from applications.
 *
 * Revision 1.15  93/08/26  15:29:31  kushi
 * added flags and return code for report_engine_error (PTAR 965)
 * (stage 1).
 *
 * Revision 1.14  93/07/20  09:54:40  degenn
 * PTAR 535.
 * At init-enrollment, add advisory space parameters (expected record,
 * expected train, actual free).  At terminate-enrollment, add required
 * space for training.
 *
 * Revision 1.13  93/06/04  15:10:45  degenn
 * David's changes for queue version: use semaphore around freeing and
 * allocation in shared memory segment, free last received message whether
 * sync or async.
 *
 * Revision 1.12  93/05/21  21:07:30  degenn
 * Many changes (dk/jku/sdg) for Sm 2.0.5 API definition: SmGet is
 * now used only as a prefix for access functions, cleanup of parameters
 * to many calls.  See documentation for full details.
 *
 * Revision 1.11  93/04/27  11:08:23  degenn
 * dk/sdg changes to drop pcm protocol, restructure spchcomm to avoid
 * having to link api + dpi + audapi + auddpi + comm.  Bumped native API
 * level for good measure.
 *
 * Revision 1.10  93/02/15  11:34:38  degenn
 * Bumped API to 2.0.3 for 2/15 drop.
 *
 * Revision 1.9  93/01/29  18:51:36  degenn
 * DK/SDG changes to native API required to support new OS/2 SM API.
 * New features include: synch/asynch flag on each request and reply,
 * so we can decide whether or not we need to "slap" the PM queue;
 * native API can call WinPostMsg to alert a PM task to unsolicited or
 * asynchronous messages; and router.msg style message logging.
 * New functions include: set/query_api_logging, set/query_pm_enable,
 * set/query_connection_id, set_synchronous, query_comm_handle/sem to
 * eliminate peeking at tp-> structure.
 * Also, set tp->commrsem semaphore when running with named pipes.
 *
 * Revision 1.8  92/12/09  22:02:46  degenn
 * Added IBM headers and rcs logging.
 *
 * Revision 1.7  92/12/09  15:47:18  daggett
 * Changes for OS/2 standalone configuration without TCP/IP.
 *
 * Revision 1.6  92/11/20  16:02:16  bergl
 * API level 1.0.3 .
 *
 * Revision 1.5  92/10/22  15:24:28  bergl
 * New results codes, increased API level to 1.0.2.
 *
 * Revision 1.4  92/10/01  19:43:08  tangora
 * JK: changed SM_ name for SPCH_RC_WRONG_SM_VERSION, dropped
 * SPCH_RC_NO_SM_HOST
 *
 * Revision 1.3  92/09/30  21:22:10  tangora
 * > Bumped API level to force all users to latest AUDCOMM on OS/2.
 * Support SM AIX work.
 *
 * Revision 1.2  92/04/27  12:04:50  sdg
 * Added BAD_LANGUAGE return code.
 *
 * Revision 1.1  92/04/21  18:02:12  sdg
 * Initial revision
 *
 */

#ifndef SPCHRC_H_INCLUDED
#define SPCHRC_H_INCLUDED

/*---------------------------------------------------------------------*/
/* Get the SM API Return codes.                                        */
/*---------------------------------------------------------------------*/
#include <smrc.h>

/*---------------------------------------------------------------------*/
/* send/receive message error codes                                    */
/*---------------------------------------------------------------------*/
#define SPCH_OK                          SM_RC_OK
#define SPCH_EAPIVERSION                 SM_RC_EAPIVERSION
#define SPCH_ENOMSG                      SM_RC_ENOMSG
#define SPCH_ENOCONN                     SM_RC_ENOCONN
#define SPCH_ENOSERVER                   SM_RC_ENOSERVER
#define SPCH_EINVAL                      SM_RC_EINVAL
#define SPCH_EUNEXP                      SM_RC_EUNEXP
#define SPCH_ETIMEOUT                    SM_RC_ETIMEOUT
#define SPCH_EUNKMSG                     SM_RC_EUNKMSG
#define SPCH_EMSGSIZE                    SM_RC_EMSGSIZE
#define SPCH_ENOHANDLES                  SM_RC_ENOHANDLES
#define SPCH_EBADHANDLE                  SM_RC_EBADHANDLE
#define SPCH_ENOMEM                      SM_RC_ENOMEM
#define SPCH_EALLOC                      SM_RC_EALLOC
#define SPCH_EBADAPPNAME                 SM_RC_EBADAPPNAME
#define SPCH_ENOACCEPT                   SM_RC_ENOACCEPT
#define SPCH_EALLBUSY                    SM_RC_EALLBUSY
#define SPCH_CREATE_MBOX_FAILED          SM_RC_CREATE_MBOX_FAILED
#define SPCH_CREATE_EVENT_SEM_FAILED     SM_RC_CREATE_EVENT_SEM_FAILED
#define SPCH_ASSOC_EVENT_SEM_FAILED      SM_RC_ASSOC_EVENT_SEM_FAILED
#define SPCH_DEALLOCATING_SH_MEM         SM_RC_DEALLOCATING_SH_MEM
#define SPCH_QUEUE_CLOSE_ERROR           SM_RC_QUEUE_CLOSE_ERROR
#define SPCH_FREE_MEM_ERROR              SM_RC_FREE_MEM_ERROR
#define SPCH_CLOSE_EVENT_SEM_FAILED      SM_RC_CLOSE_EVENT_SEM_FAILED
#define SPCH_SUB_UNSET_ERROR             SM_RC_SUB_UNSET_ERROR
#define SPCH_OPEN_QUEUE_FAILED           SM_RC_OPEN_QUEUE_FAILED
#define SPCH_CREATE_MUTEX_SEM_FAILED     SM_RC_CREATE_MUTEX_SEM_FAILED
#define SPCH_ATTACH_MUTEX_SEM_FAILED     SM_RC_ATTACH_MUTEX_SEM_FAILED

/*---------------------------------------------------------------------*/
/* Return codes.                                                       */
/*---------------------------------------------------------------------*/
#define SPCH_RC_NOT_YET                  SM_RC_NOT_YET

#define SPCH_RC_OK                       SM_RC_OK
#define SPCH_RC_NOT_VALID_REQUEST        SM_RC_NOT_VALID_REQUEST
#define SPCH_RC_BAD_MODE                 SM_RC_BAD_MODE
#define SPCH_RC_NOT_WHILE_MIC_ON         SM_RC_NOT_WHILE_MIC_ON
#define SPCH_RC_MIC_ALREADY_ON           SM_RC_MIC_ALREADY_ON
#define SPCH_RC_MIC_ALREADY_OFF          SM_RC_MIC_ALREADY_OFF
#define SPCH_RC_MIC_ON_PENDING           SM_RC_MIC_ON_PENDING
#define SPCH_RC_MIC_OFF_PENDING          SM_RC_MIC_OFF_PENDING
#define SPCH_RC_NOT_WHILE_PLAYING        SM_RC_NOT_WHILE_PLAYING

#define SPCH_RC_BAD_AUDIO                SM_RC_BAD_AUDIO
#define SPCH_RC_RECORD_OPEN_ERROR        SM_RC_RECORD_OPEN_ERROR
#define SPCH_RC_PLAY_OPEN_ERROR          SM_RC_PLAY_OPEN_ERROR
#define SPCH_RC_AUDIO_IN_USE             SM_RC_AUDIO_IN_USE
#define SPCH_RC_BAD_AUDIO_PROTOCOL       SM_RC_BAD_AUDIO_PROTOCOL
#define SPCH_RC_AUDIO_TIMEOUT            SM_RC_AUDIO_TIMEOUT
#define SPCH_RC_AUDIO_DISCONNECTED       SM_RC_AUDIO_DISCONNECTED
#define SPCH_RC_AUDIO_OVERRUN            SM_RC_AUDIO_OVERRUN
#define SPCH_RC_AUDIO_FORCED_MIC_OFF     SM_RC_AUDIO_FORCED_MIC_OFF
#define SPCH_RC_NO_MORE_AUDIO_FILES      SM_RC_NO_MORE_AUDIO_FILES

#define SPCH_RC_BAD_AP                   SM_RC_BAD_AP
#define SPCH_RC_BAD_DECO                 SM_RC_BAD_DECO

#define SPCH_RC_BAD_ADDWORD              SM_RC_BAD_ADDWORD
#define SPCH_RC_ADDED                    SM_RC_ADDED
#define SPCH_RC_NOT_ADDED                SM_RC_NOT_ADDED
#define SPCH_RC_MULTIPLE_SPELLINGS       SM_RC_MULTIPLE_SPELLINGS
#define SPCH_RC_ILLEGAL_SPELLING         SM_RC_ILLEGAL_SPELLING
#define SPCH_RC_ILLEGAL_SOUNDSLIKE       SM_RC_ILLEGAL_SOUNDSLIKE
#define SPCH_RC_MISMATCHED_ACOUSTICS     SM_RC_MISMATCHED_ACOUSTICS
#define SPCH_RC_BAD_ACOUSTICS            SM_RC_BAD_ACOUSTICS
#define SPCH_RC_SPELLING_TOO_LONG        SM_RC_SPELLING_TOO_LONG
#define SPCH_RC_ACOUSTICS_TOO_LONG       SM_RC_ACOUSTICS_TOO_LONG
#define SPCH_RC_ADDWORD_LIMIT_EXCEEDED   SM_RC_ADDWORD_LIMIT_EXCEEDED
#define SPCH_RC_EXTENDED_VOCAB_ADD       SM_RC_EXTENDED_VOCAB_ADD
#define SPCH_RC_RECORDING_REQUIRED       SM_RC_RECORDING_REQUIRED

#define SPCH_RC_SERVER_ERROR             SM_RC_SERVER_ERROR
#define SPCH_RC_SERVER_MALLOC_ERROR      SM_RC_SERVER_MALLOC_ERROR
#define SPCH_RC_SERVER_FILE_OPEN_ERROR   SM_RC_SERVER_FILE_OPEN_ERROR
#define SPCH_RC_SERVER_FILE_WRITE_ERROR  SM_RC_SERVER_FILE_WRITE_ERROR
#define SPCH_RC_SERVER_FILE_READ_ERROR   SM_RC_SERVER_FILE_READ_ERROR
#define SPCH_RC_SERVER_FILE_CLOSE_ERROR  SM_RC_SERVER_FILE_CLOSE_ERROR
#define SPCH_RC_SERVER_PROCESS_ERROR     SM_RC_SERVER_PROCESS_ERROR
#define SPCH_RC_SERVER_TERMINATED        SM_RC_SERVER_TERMINATED

#define SPCH_RC_BAD_TAG                  SM_RC_BAD_TAG
#define SPCH_RC_BAD_UTTNO                SM_RC_BAD_UTTNO
#define SPCH_RC_BAD_MESSAGE              SM_RC_BAD_MESSAGE

#define SPCH_RC_NOT_DELETED              SM_RC_NOT_DELETED
#define SPCH_RC_NOT_INVOCAB              SM_RC_NOT_INVOCAB
#define SPCH_RC_INVOCAB                  SM_RC_INVOCAB
#define SPCH_RC_BAD_VOCAB                SM_RC_BAD_VOCAB
#define SPCH_RC_MISSING_EXTERN           SM_RC_MISSING_EXTERN

#define SPCH_RC_BAD_USERID               SM_RC_BAD_USERID
#define SPCH_RC_BAD_ENROLLID             SM_RC_BAD_ENROLLID
#define SPCH_RC_BAD_PASSWORD             SM_RC_BAD_PASSWORD
#define SPCH_RC_BAD_TASKID               SM_RC_BAD_TASKID
#define SPCH_RC_BAD_CLIENT               SM_RC_BAD_CLIENT
#define SPCH_RC_USERID_EXISTS            SM_RC_USERID_EXISTS
#define SPCH_RC_ENROLLID_EXISTS          SM_RC_ENROLLID_EXISTS
#define SPCH_RC_USERID_BUSY              SM_RC_USERID_BUSY
#define SPCH_RC_ENROLLID_BUSY            SM_RC_ENROLLID_BUSY
#define SPCH_RC_BAD_SCRIPT               SM_RC_BAD_SCRIPT
#define SPCH_RC_BAD_DESCRIPTION          SM_RC_BAD_DESCRIPTION
#define SPCH_RC_ENROLLID_RUNNING         SM_RC_ENROLLID_RUNNING
#define SPCH_RC_ENROLLMENT_NOT_COMPLETE  SM_RC_ENROLLMENT_NOT_COMPLETE
#define SPCH_RC_MISMATCHED_LANGUAGE      SM_RC_MISMATCHED_LANGUAGE
#define SPCH_RC_MISMATCHED_ALPHABET      SM_RC_MISMATCHED_ALPHABET
#define SPCH_RC_MISMATCHED_SCRIPT        SM_RC_MISMATCHED_SCRIPT
#define SPCH_RC_BAD_LANGUAGE             SM_RC_BAD_LANGUAGE
#define SPCH_RC_BAD_NAME                 SM_RC_BAD_NAME
#define SPCH_RC_INVALID_WINDOW_HANDLE    SM_RC_INVALID_WINDOW_HANDLE

#define SPCH_RC_BAD_ITEM                 SM_RC_BAD_ITEM
#define SPCH_RC_BAD_VALUE                SM_RC_BAD_VALUE

#define SPCH_RC_BUSY_LAST_UTTERANCE      SM_RC_BUSY_LAST_UTTERANCE
#define SPCH_RC_BUSY_WORD_CORRECTION     SM_RC_BUSY_WORD_CORRECTION

#define SPCH_RC_NO_SPACE                 SM_RC_NO_SPACE
#define SPCH_RC_NO_SPACE_INIT_RECO       SM_RC_NO_SPACE_INIT_RECO
#define SPCH_RC_NO_SPACE_INIT_ENROLL     SM_RC_NO_SPACE_INIT_ENROLL
#define SPCH_RC_NO_SPACE_TERM_ENROLL     SM_RC_NO_SPACE_TERM_ENROLL
#define SPCH_RC_NO_SPACE_MIC_ON          SM_RC_NO_SPACE_MIC_ON

#define SPCH_RC_NO_FOCUS_APP             SM_RC_NO_FOCUS_APP
#define SPCH_RC_FOCUS_GRANTED            SM_RC_FOCUS_GRANTED
#define SPCH_RC_FOCUS_REQUEST_PENDING    SM_RC_FOCUS_REQUEST_PENDING
#define SPCH_RC_FOCUS_DENIED             SM_RC_FOCUS_DENIED
#define SPCH_RC_NAV_ALREADY_DEFINED      SM_RC_NAV_ALREADY_DEFINED

#define SPCH_RC_NOT_IN_NOTIFY            SM_RC_NOT_IN_NOTIFY
#define SPCH_RC_EXISTS_IN_NOTIFY         SM_RC_EXISTS_IN_NOTIFY

#define SPCH_RC_INCOMPATIBLE_ENROLLMENT  SM_RC_INCOMPATIBLE_ENROLLMENT

/*---------------------------------------------------------------------*/
/* Status codes.                                                       */
/*---------------------------------------------------------------------*/
#define SPCH_STAT_PLAY_START             SM_STAT_PLAY_START
#define SPCH_STAT_PLAY_STOP              SM_STAT_PLAY_STOP
#define SPCH_STAT_BAD_AUDIO              SM_STAT_BAD_AUDIO
#define SPCH_STAT_BAD_TAG                SM_STAT_BAD_TAG
#define SPCH_STAT_BAD_UTTERANCE          SM_STAT_BAD_UTTERANCE
#define SPCH_STAT_BAD_MESSAGE            SM_STAT_BAD_MESSAGE

#define SPCH_STAT_ENROLLMENT_RECORDING   SM_STAT_ENROLLMENT_RECORDING
#define SPCH_STAT_ENROLLMENT_RUNNING     SM_STAT_ENROLLMENT_RUNNING
#define SPCH_STAT_ENROLLMENT_FAILED      SM_STAT_ENROLLMENT_FAILED
#define SPCH_STAT_ENROLLMENT_COMPLETE    SM_STAT_ENROLLMENT_COMPLETE
#define SPCH_STAT_ENROLLMENT_BUSY        SM_STAT_ENROLLMENT_BUSY

/*---------------------------------------------------------------------*/
/* Speech Manager Error codes                                          */
/*---------------------------------------------------------------------*/
#define SPCH_RC_NOT_OPEN                SM_RC_SM_NOT_OPEN
#define SPCH_RC_WRONG_SM_VERSION        SM_RC_WRONG_SM_VERSION
#define SPCH_RC_NOT_ACTIVE_CLIENT       SM_RC_SM_NOT_ACTIVE_CLIENT
#define SPCH_RC_EINVAL_MSG_TYPE         SM_RC_SM_EINVAL_MSG_TYPE
#define SPCH_RC_REPLY_NULL              SM_RC_REPLY_NULL
#define SPCH_RC_NO_MORE_CONNECTIONS     SM_RC_NO_MORE_CONNECTIONS
#define SPCH_RC_NO_TOPLEVEL_WIDGET      SM_RC_NO_TOPLEVEL_WIDGET
#define SPCH_RC_CONNECTION_CHANGED      SM_RC_CONNECTION_CHANGED
#define SPCH_RC_CALLBACK_LIST_CHANGED   SM_RC_CALLBACK_LIST_CHANGED
#define SPCH_RC_ALREADY_OPENED          SM_RC_ALREADY_OPENED
#define SPCH_RC_ALREADY_CONNECTED       SM_RC_ALREADY_CONNECTED

#endif /* not SPCHRC_H_INCLUDED */
