//------------------------------------------------------------------------------
//
// COPYRIGHT:
// ----------
// Copyright (C) International Business Machines Corp., 1996.
//
// DISCLAIMER OF WARRANTIES:
// -------------------------
// The following [enclosed] code is sample code created by IBM
// Corporation.  This sample code is not part of any standard IBM
// product and is provided to you solely for the purpose of
// assisting you in the development of your applications.  The
// code is provided "AS IS", without warranty of any kind.  IBM
// shall not be liable for any damages arising out of your use of
// the sample code, even if they have been advised of the
// possibility of such damages.
//
//------------------------------------------------------------------------------
//
//  gramsel.cpp : Implements methods of the GrammarSelect class.
//                This is a Windows dialog class interacts with a user to
//                make grammar and vocabulary selections.
//
//                The dialog uses a VTProxy object which is used for all 
//                communications with the speech engine. It is the VTProxy 
//                class which implements the calls to SMAPI functions.  
//
//------------------------------------------------------------------------------
//

#include "stdafx.h"
#include "eReco.h"
#include "gramSel.h"
#include "defGram.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//------------------------------------------------------------------------------
//  GrammarSelect()
//------------------------------------------------------------------------------
//
//  MFC Generated Constructor for the GrammarSelect Windows dialog.
//
//------------------------------------------------------------------------------
GrammarSelect::GrammarSelect(VTProxy   &aVTProxy,
                             CPtrArray &aGrammarList,
							        CWnd* pParent /*=NULL*/)
	: m_pxySMAPI(aVTProxy),
     m_listGrammarDescriptors(aGrammarList),
	  CDialog(GrammarSelect::IDD, pParent)
{
	//{{AFX_DATA_INIT(GrammarSelect)
	//}}AFX_DATA_INIT
   
   toggle = FALSE;

}

GrammarSelect::~GrammarSelect()
{

}
//------------------------------------------------------------------------------
//  DoDataExchange()
//------------------------------------------------------------------------------
//
//  MFC Generated Windows data exchange method.
//
//------------------------------------------------------------------------------
void GrammarSelect::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(GrammarSelect)
	DDX_Control(pDX, IDC_LIST_VOCABS_N_GRAMMARS, m_listVocabsNGrammars);
	DDX_Control(pDX, IDC_RADIO_GRAMMAR_FILE, m_radioSelect);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(GrammarSelect, CDialog)
	//{{AFX_MSG_MAP(GrammarSelect)
	ON_BN_CLICKED(IDC_BUTTON_SELECT1, OnButtonSelect)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST_VOCABS_N_GRAMMARS, OnDblclkListVocabsNGrammars)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


//------------------------------------------------------------------------------
//  OnInitDialog()
//------------------------------------------------------------------------------
//
//  Initialize the  grammar selection dialog
//
//----------------------------------------------------------------------------
BOOL GrammarSelect::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// initialize the radio buttons
	InitRadioButtons();

   // initialize the grammar list view
   m_listVocabsNGrammars.InsertColumn(1, "Vocabulary Name", 
                         LVCFMT_LEFT, 270);
   m_listVocabsNGrammars.InsertColumn(2, "Type", 
                         LVCFMT_LEFT, 75, 1);
   m_listVocabsNGrammars.InsertColumn(3, "Selected", 
                         LVCFMT_LEFT, 60, 2);

   // display the grammar list
	ShowGrammarList();
 
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


//------------------------------------------------------------------------------
//  OnButtonSelect()
//------------------------------------------------------------------------------
//
//  Message handler for when the user presses the "DO IT!!" button
//
//----------------------------------------------------------------------------
void GrammarSelect::OnButtonSelect() 
{

	toggle = FALSE;
	switch( GetCheckedRadioButton(IDC_RADIO_GRAMMAR_FILE,
		                          IDC_RADIO_GRAMMAR_ENABLE) ) {

      case IDC_RADIO_GRAMMAR_FILE :
         // the "Select/Define Grammar" radio button was selected
         AddFileToGrammarList();
         break;

      case IDC_RADIO_DYNAMIC_LIST :
         // the "Define Dynamic Grammar" radio button was selected
		   DefineNewGrammar(dynamic);
		   break;

      case IDC_RADIO_EXTERNAL_LIST :     
          // the "Define External Grammar" radio button was selected
          DefineNewGrammar(external);
          break;
  
    }
	
}


//------------------------------------------------------------------------------
//  OnOK()
//------------------------------------------------------------------------------
//
// Message handler for when the OK button has been pushed
//
//----------------------------------------------------------------------------
void GrammarSelect::OnOK() 
{

   for (int i=0; i < m_listGrammarDescriptors.GetSize(); i++) 
   {
    	GrammarDescriptor *theGrammar = (GrammarDescriptor *)
         m_listGrammarDescriptors.GetAt(i);

      if ( theGrammar->newDefinition() )
      {
         // this is a newly defined grammar/vocabulary
         // acknowledge it
         theGrammar->acknowledgeNewDefinition();
      }      
   }
	
	CDialog::OnOK();
}

//------------------------------------------------------------------------------
//  OnCancel()
//------------------------------------------------------------------------------
//
//  Message handler for when the Cancel button is pressed
//
//----------------------------------------------------------------------------
void GrammarSelect::OnCancel() 
{
	// cleanup 
   int i=0; 
   while ( i < m_listGrammarDescriptors.GetSize() ) 
   {
    	GrammarDescriptor *theGrammar = (GrammarDescriptor *)
         m_listGrammarDescriptors.GetAt(i);

      if ( theGrammar->newDefinition() )
      {
         // this is a newly defined grammar/vocabulary
         // undefine it & clean up
         if ( theGrammar->type() == fsg ) 
         { 
            //m_pxySMAPI.undefineGrammar( theGrammar->fsgFileName() );
            m_pxySMAPI.undefineVocabulary( theGrammar->vocabularyName() );
         }
         else
         {
            m_pxySMAPI.undefineVocabulary( theGrammar->vocabularyName() );
         }

         m_listGrammarDescriptors.RemoveAt(i); // removing 
                                   // an element shifts all
                                   // following elements 
                                   // down 1 index
         delete(theGrammar);
      }
      else
      {
         // only increment if an element is NOT removed
         i++;
      }
           
   }
	
	CDialog::OnCancel();
}


//------------------------------------------------------------------------------
//  AddFileToGrammarList()
//------------------------------------------------------------------------------
//
//  Present the fsg grammar file selection dialog and
//  define the selected grammar to the speech engine
//
//----------------------------------------------------------------------------
void  GrammarSelect::AddFileToGrammarList()
{
   // Show an Open File dialog to allow the user to select 
   // a grammar fsg file name.
   CFileDialog dlgFile(TRUE, NULL, "*.fsg;*.wdl",
   OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,   
      "Grammar/Word List Files (*.fsg;*.wdl) | *.fsg;*.wdl ||");

   if (dlgFile.DoModal() == IDOK)
   {

      // create a new grammar descriptor object and
      // add it to the grammar/vocabulary list
      GrammarDescriptor *aNewGD = 
         new GrammarDescriptor( dlgFile.GetPathName() ); 

      m_listGrammarDescriptors.Add( aNewGD );
      
      // add the grammar/vocabulary description to the 
      // list box
      DisplayGrammar( aNewGD );

      if ( aNewGD->type() == fsg )
      {
         // define the grammar to the speech engine
         if ( m_pxySMAPI.defineGrammar(aNewGD->fsgFileName(),     	
                                       aNewGD->vocabularyName() ) )
         {
            // grammar is implicitly selected by the user
            aNewGD->setSelect(TRUE);
         }
         else
         {     
            CString message;
            message = "Error defining file <";
            message += aNewGD->fsgFileName();
            message += ">\n\n";
                      
            MessageBox( message, "Error defining compiled grammar.", MB_OK );
         }
      }
      else  // must be a word list
      {
         // define the word list to the speech engine
         if ( m_pxySMAPI.defineWordList(aNewGD->fsgFileName(),     	
                                       aNewGD->vocabularyName() ) )
         {
            // word list is implicitly selected by the user
            aNewGD->setSelect(TRUE);
         }
         else
         {     
            CString message;
            message = "Error defining file <";
            message += aNewGD->fsgFileName();
            message += ">\n\n";
            if ( m_pxySMAPI.isBadWordWaiting() )
            {
               message += "Use Dictionary Builder to add pronunciations for:\n";
               message += m_pxySMAPI.getBadWords( TRUE );
               message += "\n\nthese words will not be recognized.";
            }
                      
            MessageBox( message, "Error defining word list file.", MB_OK );
         }
      }
   }

   return;
}


//------------------------------------------------------------------------------
//  DefineNewGrammar()
//------------------------------------------------------------------------------
//
//  Present grammar/vocabulary definition dialog(s) to the user
//  for either a "Dynamic Grammar" or an "External Grammar" 
//  definition. Add new grammar descriptors to the grammar/vocabulary
//  list.  
//----------------------------------------------------------------------------
void  GrammarSelect::DefineNewGrammar(vocabularyTypes vocabularyType)
{

 	// present a dynamic grammar definition dialog
	DefineGrammar  definitionDialog( m_pxySMAPI); 

	int nResponse = definitionDialog.DoModal();
	if (nResponse == IDOK)
	{

      // create a new grammar descriptor and add it to the 
      // grammar/vocabulary list
      GrammarDescriptor *aNewGD =
         new GrammarDescriptor( definitionDialog.vocabularyName() ); 
      aNewGD->setType(vocabularyType);


      // vocabs are selected by default except for external lists which should
      // normally be defined (already done) but not enabled.  External lists
      // are not enabled because they are enabled as part of the grammar which
      // references them.

      aNewGD->setSelect( vocabularyType != external );  // set select/deselect
		
      m_listGrammarDescriptors.Add( aNewGD );
      
      // display the new grammar/vocabulary in the listbox
      DisplayGrammar( aNewGD );

      if (vocabularyType == external) 
      {
         // present a fsg file selection dialog to 
         // complete the external grammar definition
         char message[256];
       
         sprintf(message, 
                 "Select the fsg file which contains the external attribute <%s>\n",
                  definitionDialog.vocabularyName() );

         MessageBox( message,
                     "Next Step is... ",
                      MB_OK);

         AddFileToGrammarList();

      }
 
	}
	else if (nResponse == IDCANCEL)
	{
		
      // TODO: Place code here to handle when the dialog is
		//  dismissed with Cancel
	}


}


//------------------------------------------------------------------------------
//  OnDblclkListVocabsNGrammars()
//------------------------------------------------------------------------------
//
//  An entry in the grammar/vocabulary list box has been selected..
//  Ungray the select/deselect radio button
//
//----------------------------------------------------------------------------
void GrammarSelect::OnDblclkListVocabsNGrammars(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
   CString   selectedName;

   DWORD dwPos = ::GetMessagePos();
   CPoint point( (int)LOWORD(dwPos), (int)HIWORD(dwPos) );
   m_listVocabsNGrammars.ScreenToClient(&point);

   int nIndex;
   if ( ( nIndex = m_listVocabsNGrammars.HitTest(point) ) != -1 )
   {
      selectedName = m_listVocabsNGrammars.GetItemText(nIndex, 0);
   }

   // look for the selected grammar/vocabulary in the 
   // grammar list
   for (int i = 0; i < m_listGrammarDescriptors.GetSize(); i++)
   {
     	// look for the named grammar/vocabulary in the listBox
      // entry
      GrammarDescriptor *theGrammar = (GrammarDescriptor *)
                             m_listGrammarDescriptors.GetAt(i);
      //int rc = selectedListEntry.Find(theGrammar->fsgFileName());
      //if (rc != -1)
      if ( selectedName == theGrammar->fsgFileName() )
      {
         // grammar/vocabulary was found... toggle it
         if ( theGrammar->selected() )
         {
            theGrammar->setSelect(FALSE);
         }
         else
         {
            theGrammar->setSelect(TRUE);
         }
      }

      ShowGrammarList();
   }

	*pResult = 0;
}


//------------------------------------------------------------------------------
//  InitRadioButtons()
//------------------------------------------------------------------------------
//
//  Set the radio buttons to a default or initialization state
//
//----------------------------------------------------------------------------
void GrammarSelect::InitRadioButtons() 
{
	// initialize the radio buttons to a default state
   CheckRadioButton(IDC_RADIO_GRAMMAR_FILE,
                    IDC_RADIO_GRAMMAR_ENABLE,
   		           IDC_RADIO_GRAMMAR_FILE);
	
}

//------------------------------------------------------------------------------
//  ShowGrammarList()
//------------------------------------------------------------------------------
//
//  Show the entire grammar/vocabulary list in the grammar/vocabulary
//  listbox
//
//----------------------------------------------------------------------------
void GrammarSelect::ShowGrammarList() 
{
   // initialize grammar selection list box
   m_listVocabsNGrammars.DeleteAllItems();
   InitRadioButtons();

   for (int i=0; i < m_listGrammarDescriptors.GetSize(); i++) 
   {
    	DisplayGrammar( (GrammarDescriptor *)
		                m_listGrammarDescriptors.GetAt(i) );
   }
	
}

//------------------------------------------------------------------------------
//  DisplayGrammar()
//------------------------------------------------------------------------------
//
//  Add a single grammar/vocabulary to be displayed in the 
//  grammar/vocabulary listbox
//
//----------------------------------------------------------------------------
void GrammarSelect::DisplayGrammar(GrammarDescriptor *theGrammar)
{

   CString  selectString;

   if (theGrammar->selected() )
   {
      selectString = "yes";
   }
   else
   {
      selectString = "no";
   }

   int idx = m_listVocabsNGrammars.InsertItem(0, theGrammar->fsgFileName());
   m_listVocabsNGrammars.SetItemText(idx, 1, theGrammar->typeString());
   m_listVocabsNGrammars.SetItemText(idx, 2, selectString);

}


