//------------------------------------------------------------------------------
//
// COPYRIGHT:
// ----------
// Copyright (C) International Business Machines Corp., 1997.
//
// DISCLAIMER OF WARRANTIES:
// -------------------------
// The following [enclosed] code is sample code created by IBM
// Corporation.  This sample code is not part of any standard IBM
// product and is provided to you solely for the purpose of
// assisting you in the development of your applications.  The
// code is provided "AS IS", without warranty of any kind.  IBM
// shall not be liable for any damages arising out of your use of
// the sample code, even if you have been advised of the
// possibility of such damages.
//
//------------------------------------------------------------------------------
// 
// This is a multilingual kiosk application
// 
// A simple dialog is presented to the user and allows him to
// ask questions (e.g. 'How do I get to Times Square') and the
// system responds accordingly.
// The user can also switch languages by saying the name of another
// town (e.g. 'Vienna') - both the user interface language and
// the speech recognition language will be switched accordingly, and
// the user can continue to ask questions.
//
// The sample illustrates how to write multilingual speech applications,
// how to structure the language-dependent and independent files,
// how to load resource dll's and how to switch both the user interface
// and the speech recognition engine to another language.
//
// At runtime, the sample uses the following files:
//      (## stands for the 2-character language code,
//       us,uk,gr,fr for us/uk english,german and french)
//
// kiosk.exe    ... the (language-independent) executable;
//                  build it by opening 'kiosk.mak'
// kiosk##.dll  ... the language-specific resource dll's, 
//                  containing translated strings for each language;
//                  build them by opening 'kiosk##.mak'
// kiosk##.fsg  ... the compiled speech grammars for each language;
//                  build them with the grammar compiler, the sources
//                  are kiosk##.bnf.
// kiosk##.dat  ... the data files for the questions and answers for
//                  each language 
// kiosk##.ppl  ... the vocabulary files for each language, containing
//                  the words in the grammars (build these files 
//                  with the Dictionary Builder; these files are 
//                  copied to the appropriate directory to be picked
//                  up by the recognition engine.
//
// Limitations:
//    - the sample shows the multilingual aspects only if 
//      multiple Voicetype Runtimes in different languages are 
//      installed and if there are enrollments for these languages.
//    - this sample cannot run with other speech applications, so
//      close all other running speech apps.
//    - only limited error checking is done
//    
//------------------------------------------------------------------------------

//
// kiosk.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "kiosk.h"
#include "kioskDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CKioskApp

BEGIN_MESSAGE_MAP(CKioskApp, CWinApp)
	//{{AFX_MSG_MAP(CKioskApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKioskApp construction

CKioskApp::CKioskApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CKioskApp object

CKioskApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CKioskApp initialization

BOOL CKioskApp::InitInstance()
{
	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	CKioskDlg dlg;
	m_pMainWnd = &dlg;
	int nResponse = dlg.DoModal();
	if (nResponse == IDOK)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with OK
	}
	else if (nResponse == IDCANCEL)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with Cancel
	}

	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	return FALSE;
}
