/******************************************************************************/
/*                                                                            */
/* COPYRIGHT:                                                                 */
/* ----------                                                                 */
/* Copyright (C) International Business Machines Corp., 1996.                 */
/*                                                                            */
/* DISCLAIMER OF WARRANTIES:                                                  */
/* -------------------------                                                  */
/* The following [enclosed] code is sample code created by IBM                */
/* Corporation.  This sample code is not part of any standard IBM product     */
/* and is provided to you solely for the purpose of assisting you in the      */
/* development of your applications.  The code is provided "AS IS",           */
/* without warranty of any kind.  IBM shall not be liable for any damages     */
/* arising out of your use of the sample code, even if they have been         */
/* advised of the possibility of such damages.                                */
/*                                                                            */
/******************************************************************************/
// recoDlg.cpp : implementation file
//

#include "stdafx.h"
#include "reco.h"
#include "recoDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRecoDlg dialog

CRecoDlg::CRecoDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CRecoDlg::IDD, pParent)
{
        //{{AFX_DATA_INIT(CRecoDlg)
        m_szFSG = _T("");
        m_phrase = _T("");
        //}}AFX_DATA_INIT
        // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
        m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
        m_szActiveGrammar[0] = '\0';
        m_szActiveVocab[0] = '\0';
        m_bRecognizing = FALSE;
        m_bEngineReady = FALSE;
}

void CRecoDlg::DoDataExchange(CDataExchange* pDX)
{
        CDialog::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CRecoDlg)
        DDX_Control(pDX, IDC_LIST_STATUS, m_listStatus);
        DDX_Control(pDX, IDC_EDIT_FSG_FILE, m_editFSG);
        DDX_Control(pDX, IDC_BUTTON_START_RECO, m_buttonStartReco);
        DDX_Control(pDX, IDC_BUTTON_SELECT_FSG, m_buttonSelectFSG);
        DDX_Control(pDX, IDC_BUTTON_LOAD_FSG, m_buttonLoadFSG);
        DDX_Text(pDX, IDC_EDIT_FSG_FILE, m_szFSG);
        DDX_Text(pDX, IDC_reC_phrase, m_phrase);
        DDV_MaxChars(pDX, m_phrase, 1024);
        //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CRecoDlg, CDialog)
        //{{AFX_MSG_MAP(CRecoDlg)
        ON_WM_PAINT()
        ON_WM_QUERYDRAGICON()
        ON_BN_CLICKED(IDC_BUTTON_LOAD_FSG, OnButtonLoadFsg)
        ON_BN_CLICKED(IDC_BUTTON_SELECT_FSG, OnButtonSelectFsg)
        ON_BN_CLICKED(IDC_BUTTON_START_RECO, OnButtonStartReco)
        ON_EN_UPDATE(IDC_EDIT_FSG_FILE, OnUpdateEditFsgFile)
        ON_WM_ACTIVATEAPP()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRecoDlg message handlers

BOOL CRecoDlg::OnInitDialog()
{
        CDialog::OnInitDialog();

        // Set the icon for this dialog.  The framework does this automatically
        //  when the application's main window is not a dialog
        SetIcon(m_hIcon, TRUE);                 // Set big icon
        SetIcon(m_hIcon, FALSE);                // Set small icon

        long rc = StartRecoEngine();

        if (rc == SM_RC_OK)
        {
                Display("Connecting to recognition engine.  Please wait...");
        }
        else
        {
                Display("Unable to connect to engine, rc=%d (%s)", rc, SmReturnRcDescription(rc));
        }

        m_buttonStartReco.EnableWindow(FALSE);
        m_buttonSelectFSG.EnableWindow(FALSE);
        m_buttonLoadFSG.EnableWindow(FALSE);

        UpdateData(FALSE);

        return TRUE;  // return TRUE  unless you set the focus to a control
}

//------------------------------------------------------------------------
// OnButtonLoadFsg()
//------------------------------------------------------------------------
// Loads the specified FSG file.
//------------------------------------------------------------------------
void CRecoDlg::OnButtonLoadFsg()
{
        // For the purposes of this sample, only one grammar will be active at any given
        // moment.  If the selected FSG is the same one as is currently active, then it
        // will not be redefined and reenabled.  If the name is different, then disable
        // the current grammar, then define and enable the new one.

        if (UpdateData())
        {
                m_szFSG.TrimRight();
                m_szFSG.TrimLeft();

                if (stricmp(m_szFSG, m_szActiveGrammar) != 0)
                {
                        if (DisableGrammar() == SM_RC_OK)
                        {
                                if (EnableGrammar() == SM_RC_OK)
                                {
                                        m_buttonStartReco.EnableWindow(TRUE);
                                        Display("*** Grammar <%s> now active ***", m_szActiveVocab);
                                }
                        }
                }
                else
                {
                        Display("No action taken, the grammar is already active.");
                }

                UpdateData(FALSE);
        }
}

//------------------------------------------------------------------------
// OnButtonSelectFsg()
//------------------------------------------------------------------------
// Displays file dialog for the user to select an FSG file.
//------------------------------------------------------------------------
void CRecoDlg::OnButtonSelectFsg()
{
        if (UpdateData())
        {
                // Show an Open File dialog to allow the user to select a map file name.
                CFileDialog dlgFile(TRUE, NULL, "*.fsg",
                                        OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                                        "Grammar Files (*.fsg) | *.fsg ||");

                if (dlgFile.DoModal() == IDOK)
                {
                        // Save the selected file name.
                        m_szFSG = dlgFile.GetPathName();
                        UpdateData(FALSE);
                        m_buttonLoadFSG.EnableWindow(TRUE);
                }
        }
}

//------------------------------------------------------------------------
// OnButtonStartReco()
//------------------------------------------------------------------------
// If already recognizing, turns off the microphone and stops the
// recognition process, else, turns on the microphone and starts the
// recognition process.
//------------------------------------------------------------------------
void CRecoDlg::OnButtonStartReco()
{
        if (m_bRecognizing)
        {
                if (StopRecognizing() == SM_RC_OK)
                {
                        Display("Recognition stopped, microphone turned off.  Press 'Start Recognition' to begin again.");
                        m_buttonStartReco.SetWindowText("Start &Recognizing");
                }
        }
        else
        {
                if (StartRecognizing() == SM_RC_OK)
                {
                        Display("Recognition started, microphone turned on.");
                        m_buttonStartReco.SetWindowText("Stop &Recognizing");
                }
        }

        UpdateData(FALSE);
}

//------------------------------------------------------------------------
// OnCancel()
//------------------------------------------------------------------------
// Shuts down the program in response to a cancel.
//------------------------------------------------------------------------
void CRecoDlg::OnCancel()
{
        // Close the connection to the recognition engine.
        long rc = StopRecoEngine();

        if (rc != SM_RC_OK)
        {
                CString szMsg;
                szMsg.Format("Unable to close connection to engine, rc=%d (%s)", rc,
                                                SmReturnRcDescription(rc));
                AfxMessageBox(szMsg);
        }

        CDialog::OnCancel();
}

//------------------------------------------------------------------------
// OnCommand()
//------------------------------------------------------------------------
// Messages from the engine come to the application as WM_COMMAND messages,
// identified by having the low order word of the WPARAM set to the
// connection identifier specified on the SmConnect() API call.
//------------------------------------------------------------------------
BOOL CRecoDlg::OnCommand(WPARAM wParam, LPARAM lParam)
{
   if (LOWORD(wParam) == MY_CONNECTION_ID)
   {
      return ProcessEngineMessage(lParam);
   }
   else if (LOWORD(wParam) == ID_HELP)
   {
      MessageBox( "No Help available for this sample program" );
      return TRUE;
   }

   return CDialog::OnCommand(wParam, lParam);
}

//------------------------------------------------------------------------
// OnUpdateEditFsgFile()
//------------------------------------------------------------------------
// Enables or disables the 'Load FSG' button based on whether or not there
// is a value in the FSG entry field.
//------------------------------------------------------------------------
void CRecoDlg::OnUpdateEditFsgFile()
{
        if (UpdateData())
        {
                m_szFSG.TrimRight();
                m_szFSG.TrimLeft();

                if (m_szFSG.IsEmpty())
                {
                        m_buttonLoadFSG.EnableWindow(FALSE);
                }
                else
                {
                        m_buttonLoadFSG.EnableWindow(TRUE);
                }
        }
}

//------------------------------------------------------------------------
// OnActivateApp()
//------------------------------------------------------------------------
// App is not active, make sure we have the speech focus.
//------------------------------------------------------------------------
void CRecoDlg::OnActivateApp(BOOL bActive, HTASK hTask)
{
        CDialog::OnActivateApp(bActive, hTask);

        SM_MSG reply;
        int rc;

        if (bActive && m_bEngineReady)
        {
                rc = SmRequestFocus(&reply);

                if (rc != SM_RC_OK && rc != SM_RC_FOCUS_GRANTED)
                {
                        Display("Error on SmRequestFocus: %s, rc=%d", SmReturnRcDescription(rc), rc);
                }
        }
}



//------------------------------------------------------------------------
// Functions which handle the interface to the recognition engine follow.
//------------------------------------------------------------------------

//------------------------------------------------------------------------
// StartRecoEngine()
//------------------------------------------------------------------------
// This function initializes the application's connection to the recognition
// engine.
//------------------------------------------------------------------------
long CRecoDlg::StartRecoEngine()
{
        long rc;
        char *szVersion;

        rc = SmApiVersionCheck(SM_API_VERSION_STRING, &szVersion);

        if (rc == SM_RC_OK)
        {
                rc = SmOpen(0, NULL);

                if (rc == SM_RC_OK)
                {
                        int nNumArgs = 0;
                        SmArg smArgs[30];

                        SmSetArg(smArgs[nNumArgs], SmNapplicationName, "reco");                 nNumArgs++;
                        SmSetArg(smArgs[nNumArgs], SmNrecognize, TRUE);                                 nNumArgs++;
                        SmSetArg(smArgs[nNumArgs], SmNnavigator, FALSE);                                nNumArgs++;
                        SmSetArg(smArgs[nNumArgs], SmNwindowHandle, m_hWnd);                    nNumArgs++;
                        SmSetArg(smArgs[nNumArgs], SmNconnectionId, MY_CONNECTION_ID);  nNumArgs++;

                        rc = SmConnect(nNumArgs, smArgs, SmAsynchronous);
                }
        }

        return rc;
}

//------------------------------------------------------------------------
// StopRecoEngine()
//------------------------------------------------------------------------
// This function closes the application's connection to the recognition
// engine.
//------------------------------------------------------------------------
long CRecoDlg::StopRecoEngine()
{
        SM_MSG reply;
        long  rc = 0;

        rc = SmDisconnect(0, NULL, &reply);

        if ( rc == SM_RC_OK )
        {
                m_bEngineReady = FALSE;
                rc = SmClose();
        }

        return rc;
}

//------------------------------------------------------------------------
// ProcessEngineMessage()
//------------------------------------------------------------------------
// Handles messages from the engine.
//------------------------------------------------------------------------
BOOL CRecoDlg::ProcessEngineMessage(LPARAM lParam)
{
        SM_MSG reply;
        int rc;
        int nMessageType;

        SmReceiveMsg (lParam, &reply);
        SmGetMsgType (reply, &nMessageType);
        SmGetRc(reply, &rc);

        switch (nMessageType)
        {
                case SM_CONNECT_REPLY:
                        if (rc == SM_RC_OK)
                        {
                                Display("Connected to engine, please select a grammar.");
                                m_buttonSelectFSG.EnableWindow(TRUE);
                                m_bEngineReady = TRUE;
                        }
                        else
                        {
                                Display("Error on SM_CONNECT_REPLY: %s, rc=%d", SmReturnRcDescription(rc), rc);
                        }

                        break;

                case SM_FOCUS_GRANTED:
                        Display("Got SM_FOCUS_GRANTED");
                        if (m_bRecognizing)
                        {
                                BeginListening();
                        }

                        break;

                case SM_FOCUS_LOST:
                        Display("Got SM_FOCUS_LOST");
                        break;

                case SM_RECOGNIZED_PHRASE:
                        Display("Got SM_RECOGNIZED_PHRASE");
                        if (rc == SM_RC_OK)
                        {
                                ProcessRecognizedPhrase(reply);
                        }
                        else
                        {
                                Display("Error on SM_RECOGNIZED_PHRASE: %s, rc=%d", SmReturnRcDescription(rc), rc);
                        }

                        break;

                case SM_RECOGNIZED_WORD:
                        Display("Got SM_RECOGNIZED_WORD");
                        rc = SmRecognizeNextWord(&reply);
   
                        if (rc != SM_RC_OK)
                        {
                             Display("Error on SmRecognizeNextWord: %s, rc=%d", SmReturnRcDescription(rc), rc);
                        }
                        break;

                case SM_RECOGNIZED_TEXT:
                        Display("Got SM_RECOGNIZED_TEXT");
                        break;

                case SM_REQUEST_MIC_ON:
                        Display("Got SM_REQUEST_MIC_ON");
                        // Is the microphone on?
                        if (!m_bRecognizing)
                        {
                                // No, turn it on.
                                OnButtonStartReco();
                        }
                        break;

                case SM_REQUEST_MIC_OFF:
                        Display("Got SM_REQUEST_MIC_OFF");
                        // Is the microphone on?
                        if (m_bRecognizing)
                        {
                                // Yes, turn it off.
                                OnButtonStartReco();
                        }
                        break;

                case SM_UTTERANCE_COMPLETED:
                        Display("*** Got SM_UTTERANCE_COMPLETED ***");
                        break;

                case SM_REPORT_ENGINE_ERROR:
                        ProcessEngineError(reply, rc);
                        break;

                default:
                {
                        Display("*** Received unexpected SM message: %d ***", nMessageType);
                        break;
                }
        }

        UpdateData(FALSE);
        return TRUE;
}


//------------------------------------------------------------------------
// ProcessRecognizedPhrase()
//------------------------------------------------------------------------
// Handles the phrase recognized by the engine.
//------------------------------------------------------------------------
int CRecoDlg::ProcessRecognizedPhrase(SM_MSG reply)
{
        SM_WORD* pWords;
        ULONG nNumWords, nFlags;
        SM_MSG replySync;

        int rc = SmGetFirmWords(reply, &nNumWords, &pWords);

        if (rc == SM_RC_OK)
        {
                rc = SmGetPhraseState(reply, &nFlags);

                if (rc == SM_RC_OK)
                {
                        CString szRecoText = _T("");

                        if (nFlags & SM_PHRASE_REJECTED)
                        {
                                szRecoText += "??";
                        }

                        ULONG nCounter = nNumWords;

                        while (nCounter > 0)
                        {

                                szRecoText += " ";
                                szRecoText +=(LPCTSTR)pWords->spelling ;

                                pWords++;
                                nCounter--;
                        }
                        Display("Recognized: <%s>", (LPCTSTR)szRecoText);
                        m_phrase = szRecoText;
                        UpdateData(FALSE);

                        rc = SmRecognizeNextWord(&replySync);

                        if (rc != SM_RC_OK)
                        {
                                Display("Error on SmRecognizeNextWord: %s, rc=%d", SmReturnRcDescription(rc), rc);
                        }
                }
                else
                {
                        Display("Error on SmGetPhraseState: %s, rc=%d", SmReturnRcDescription(rc), rc);
                }
        }
        else
        {
                Display("Error on SmGetFirmWords: %s, rc=%d", SmReturnRcDescription(rc), rc);
        }

        return rc;
}

//------------------------------------------------------------------------
// ProcessEngineError()
//------------------------------------------------------------------------
// Handles errors reported by the engine (basically, displays a message).
//------------------------------------------------------------------------
int CRecoDlg::ProcessEngineError(SM_MSG reply, int rcEngine)
{
        ULONG nSeverity;
        CString szMsg;

        int rc = SmGetSeverity(reply, &nSeverity);

        if (rc == SM_RC_OK)
        {
                switch (nSeverity)
                {
                        case SM_ENGINE_INFO:
                                Display("*** SM_ENGINE_INFO: %d ***", rcEngine);
                                break;

                        case SM_ENGINE_WARNING:
                                Display("*** SM_ENGINE_WARNING: %d ***", rcEngine);
                                break;

                        case SM_ENGINE_ERROR:
                                Display("*** SM_ENGINE_ERROR: %d ***", rcEngine);
                                break;

                        case SM_ENGINE_TERMINAL_ERROR:
                                Display("*** SM_ENGINE_TERMINAL_ERROR: %d ***", rcEngine);
                                break;

                        default:
                                Display("*** UNKNOWN SEVERITY: %d ***", rcEngine);
                                break;
                }

        }
        else
        {
                Display("*** SM_REPORT_ENGINE_ERROR SmGetSeverityFailed, rc=%d", rc);
        }

        return rc;
}

//------------------------------------------------------------------------
// DisableGrammar()
//------------------------------------------------------------------------
// Disables the currently defined grammars.
//------------------------------------------------------------------------
int CRecoDlg::DisableGrammar()
{
        SM_MSG reply;
        int rc = SM_RC_OK;

        if (strlen(m_szActiveVocab) > 0)
        {
                rc = SmDisableVocab(m_szActiveVocab, &reply);

                if (rc == SM_RC_OK)
                {
                        m_szActiveVocab[0] = '\0';
                        m_szActiveGrammar[0] = '\0';
                }
                else
                {
                        Display("Error on SmDisableVocab: %s, rc=%d", SmReturnRcDescription(rc), rc);
                }
        }

        return rc;
}

//------------------------------------------------------------------------
// EnableGrammar()
//------------------------------------------------------------------------
// Defines and enables the current grammar file.
//------------------------------------------------------------------------
int CRecoDlg::EnableGrammar()
{
        SM_MSG reply;
        int rc;
        BOOL bVocabAlreadyDefined = FALSE;

        // Split out the file name to be used as the vocabulary name
        _splitpath(m_szFSG, NULL, NULL, m_szActiveVocab, NULL);
        strlwr(m_szActiveVocab);
        strcpy(m_szActiveGrammar, m_szFSG);

        // First see if this vocab is already defined.
        rc = IsVocabDefined(&bVocabAlreadyDefined);

        if (rc == SM_RC_OK)
        {
                if (!bVocabAlreadyDefined)
                {
                        // Grammar is not defined...define it.
                        rc = SmDefineGrammar(m_szActiveVocab, m_szActiveGrammar, 0, &reply);

                        if (rc != SM_RC_OK)
            {
                           Display("Error on SmDefineGrammar: %s, rc=%d", SmReturnRcDescription(rc), rc);
                        }
                }

                // Now enable the grammar
                if (rc == SM_RC_OK)
                {
                        rc = SmEnableVocab(m_szActiveVocab, &reply);

                        if (rc != SM_RC_OK)
                        {
                                Display("Error on SmEnableVocab: %s, rc=%d", SmReturnRcDescription(rc), rc);
                        }
                }
        }

        // Reset active grammar and vocab on error
        if (rc != SM_RC_OK)
        {
                m_szActiveVocab[0] = '\0';
                m_szActiveGrammar[0] = '\0';
        }

        return rc;
}

//------------------------------------------------------------------------
// IsVocabDefined()
//------------------------------------------------------------------------
// Determines whether or not the currently vocab is already defined.
//------------------------------------------------------------------------
int CRecoDlg::IsVocabDefined(BOOL* bResult)
{
        SM_MSG reply;
        int rc;
        ULONG nNumVocabs;
        char** pVocabs;

        *bResult = FALSE;

        rc = SmQueryVocabs(&reply);

        if (rc == SM_RC_OK)
        {
                rc = SmGetVocabList(reply, &nNumVocabs, &pVocabs);

                if (rc == SM_RC_OK)
                {
                        for (UINT i = 0; i < nNumVocabs && !(*bResult); i++)
                        {
                                if (strcmp(m_szActiveVocab, pVocabs[i]) == 0)
                                {
                                        *bResult = TRUE;
                                }
                        }
                }
                else
                {
                        Display("Error on SmGetVocabList: %s, rc=%d", SmReturnRcDescription(rc), rc);
                }
        }
        else
        {
                Display("Error on SmQueryVocabs: %s, rc=%d", SmReturnRcDescription(rc), rc);
        }

        return rc;
}

//------------------------------------------------------------------------
// StartRecognizing()
//------------------------------------------------------------------------
// Initiates recognition processing
//------------------------------------------------------------------------
int CRecoDlg::StartRecognizing()
{
        SM_MSG reply;
        int rc;

        rc = SmRequestFocus(&reply);

        switch (rc)
        {
                case SM_RC_FOCUS_GRANTED:
                        // Already have focus, go ahead and start listening.
                        rc = BeginListening();

                        if (rc == SM_RC_OK)
                        {
                                m_bRecognizing = TRUE;
                        }

                        break;

                case SM_RC_OK:
                        // Will begin listening after SM_FOCUS_GRANTED arrives.
                        m_bRecognizing = TRUE;
                        break;

                default:
                        Display("Error on SmRequestFocus: %s, rc=%d", SmReturnRcDescription(rc), rc);
        }

        return rc;
}

//------------------------------------------------------------------------
// StopRecognizing()
//------------------------------------------------------------------------
// Terminates recognition processing
//------------------------------------------------------------------------
int CRecoDlg::StopRecognizing()
{
        SM_MSG reply;
        int rc;

        rc = SmMicOff(&reply);

        if (rc == SM_RC_OK || rc == SM_RC_MIC_ALREADY_OFF)
        {
                m_bRecognizing = FALSE;
        }
        else
        {
                Display("Error on SmMicOff: %s, rc=%d", SmReturnRcDescription(rc), rc);
        }

        return rc;
}

//------------------------------------------------------------------------
//  BeginListening()
//------------------------------------------------------------------------
//  Starts recognition processing
//------------------------------------------------------------------------
int CRecoDlg::BeginListening()
{
        SM_MSG reply;

        int rc = SmMicOn(&reply);

        if (rc == SM_RC_OK || rc == SM_RC_MIC_ALREADY_ON)
        {
                rc = SmRecognizeNextWord(&reply);

                if (rc != SM_RC_OK)
                {
                        Display("Error on SmRecognizeNextWord: %s, rc=%d", SmReturnRcDescription(rc), rc);
                }
        }
        else
        {
                Display("Error on SmMicOn: %s, rc=%d", SmReturnRcDescription(rc), rc);
        }

        return rc;
}

//------------------------------------------------------------------------
//  Display()
//------------------------------------------------------------------------
//  prints messages to the user
//------------------------------------------------------------------------

void CRecoDlg::Display(const char* pszFormat, ...)
{
        char szLine[256];
        va_list vaArgList;

        va_start(vaArgList, pszFormat);
        vsprintf(szLine, pszFormat, vaArgList);
        va_end(vaArgList);

        int rc = m_listStatus.SetCurSel(m_listStatus.AddString(szLine));
}


